/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import ij.*;
import ij.io.*;
import ij.gui.*;
import ij.Prefs;
import ij.text.*;
import ij.plugin.*;

public class Catalog_ implements PlugIn {

	static final String VERSION = "1.00";
	static final int BUTTONS = 10;
	static Frame instance;
	Panel samplePanel, musclePanel, stainingPanel, imagePanel, measurementPanel;
	TextField tRabbit, tImage;
	String imageName = "Dummy";
	ImageJ ij;
	Rabbit [] rabbits = new Rabbit[20];
	MuscleType [] muscleTypes = new MuscleType[50];
	int rabbitCount = 0;
	
	int muscleTypeCount = 0;
	TextTableWindow mw;
	TextTableWindow sw;
//	TextWindow mw;
//	TextWindow sw;
	
	String mfdbDir = "";
	String mfdbName = "MFdatabase.txt";
	String mfImgDir = "";
	String mfIntDir = "";
	String rlist = "";
	String mtlist = "";
	
	String rabCols = "Name"
		+ "\t" + "Description"
		+ "\t" + "Date"
		+ "\t" + "Procedure"
		;

	String muscleTypeCols = "Name"
						+ "\t" + "Location"
						+ "\t" + "Type"
						+ "\t" + "% Slow"
						+ "\t" + "Nebulin kDa"
						+ "\t" + "Nebulin Error"
						+ "\t" + "Titin kDa"
						+ "\t" + "Titin Error"
						+ "\t" + "Reported Length"
						;
	
	public void run(String arg) {
		// Make action window
		CatalogActions ca = new CatalogActions("Actions");
		ca.setCatalog(this);
	}

	public void openMFBD () {
		mfdbDir = Prefs.get("MFDB.Directory","");
		if (mfdbDir.equals("")) {
			// MF Database location not stored in preferences, find a pre-existing one or make a new one
			if (!findDatabase()) makeDatabase();
			Prefs.set("MFDB.Directory",mfdbDir);
		}
		System.out.println("Catalog_.run: mfdbDir= "+mfdbDir);
		
		// Open Database File
		Opener o = new Opener();
		o.open(mfdbDir + mfdbName);
		
		// Make database window
		String headings = "Record"
							+ "\t" + "Sample"
							+ "\t" + "Muscle"
							+ "\t" + "Experiment"
							+ "\t" + "ImageFile"
							+ "\t" + "Measurement"
							;
		TextWindow tw = new TextWindow("Myofibril Catalog",headings,"",500,500);
		TextPanel tp = tw.getTextPanel();
		
		// Make muscle list
		String a = "";
		int na = 0;
		if (this.muscleTypes != null) na = this.muscleTypeCount;
		for (int i=0; i<na; i++) {
			a = a + this.muscleTypes[i].name;
			if (i<na-1) a = a + "\n";
		}
		mw = new TextTableWindow("Muscle Types",a,300,500);
//		mw = new TextWindow("Muscle Types",a,300,500);
		
		// Make sample list
		String b = "";
		int nb = this.rabbitCount;
		for (int i=0; i<nb; i++) {
			b = b + this.rabbits[i].name;
			if (i<nb-1) b = b + "\n";
		}
		sw = new TextTableWindow("Sample Types",b,200,500);
//		sw = new TextWindow("Sample Types",b,200,500);
	}

	public boolean findDatabase() {
		
		// Choose a directory
		DirectoryChooser dc = new DirectoryChooser("Make a new folder for a database");
		this.mfdbDir = dc.getDirectory();
		File file = new File(this.mfdbDir);
		System.out.println("Catalog_.makeNewDatabase: mfdbDir= "+mfdbDir);
		
		// Get directory contents
		String[] contents = file.list();
		
		// If directory is empty, this is not a pre-existing database
		if (contents.length == 0) {
			System.out.println("Database folder is empty");
			return false;
		}
		
		// If directory contains a database file, this is a pre-existing database
		for (int i=0; i<contents.length; i++) {
			if (contents[i].equals(this.mfdbName)) return true;
		}
		return false;
	}
	
	public void makeDatabase() {
		// Make Linescan Image Folder
		this.mfImgDir = this.mfdbDir + "images" + "\\";
		File imgDir = new File(this.mfImgDir);
		imgDir.mkdir();
		
		// Make Intensity Folder
		this.mfIntDir = this.mfdbDir + "intensities" + "\\";
		File intDir = new File(this.mfIntDir);
		intDir.mkdir();
	}
	
	public void addRabbit() {
		GenericDialog gd = new GenericDialog("Add Rabbit");
		String newRabbitName = "Rabbit " + (this.rabbitCount + 1);
		gd.addStringField("Name",newRabbitName,20);
		String newRabbitDesc = "description";
		gd.addStringField("Description",newRabbitDesc,20);
		String newRabbitDate = "01/01/01";
		gd.addStringField("Date (mm/dd/yy)",newRabbitDate,20);
		String procedure = "dissection";
		gd.addStringField("Procedure",procedure,20);
		gd.showDialog();
		newRabbitName = gd.getNextString();
		newRabbitDesc = gd.getNextString();
		newRabbitDate = gd.getNextString();
		procedure = gd.getNextString();
		System.out.println("Catalog_.addRabbit: entry= "+newRabbitName+", "+newRabbitDesc+", "+newRabbitDate+
				", "+procedure);
		this.rabbits[this.rabbitCount] = new Rabbit(newRabbitName,newRabbitDesc,newRabbitDate, procedure);
		sw = openRabbitWindow();
		sw.getTextPanel().setVisible(false);
		sw.append(rabbits[rabbitCount].tabOut());
		this.rabbitCount++;
	}
	
	public void saveRabbitList() {
		
	}
	
	public void openRabbitList() {
		String path = Prefs.get("MFDB.rabbitlist","");
		OpenDialog od = new OpenDialog("Open Rabbit List",path);
		String dir = od.getDirectory();
		String filename = od.getFileName();
		sw = new TextTableWindow(dir+filename,300,500);
//		sw = new TextWindow(path,300,500);
		
		TextTablePanel tp = sw.getTextTablePanel();
//		TextPanel tp = sw.getTextPanel();
		int lines = tp.getLineCount();
//		System.out.println("Catalog_.openRabbitList: lines= "+lines);
		String[] data = new String[lines];
		for (int i=0; i<lines; i++) {
			data[i] = tp.getLine(i);
//			System.out.println("Catalog_.openRabbitList: data["+i+"]:"+data[i]);
		}
		sw.getTextPanel().setVisible(false);
//		System.out.println("Catalog_.openRabbitList: supertpheadings= "+sw.getTextPanel().getColumnHeadings());
		tp.setColumnHeadings(data[0]);
		for (int i=1; i<lines; i++) {
			tp.appendLine(data[i]);
		}
		Prefs.set("MFDB.rabbitlist",path);
		
	}
	
	public void openMuscleTypeList() {
		String path = Prefs.get("MFDB.muscletypelist","");
		OpenDialog od = new OpenDialog("Open MuscleType List",path);
		String dir = od.getDirectory();
		String filename = od.getFileName();
		mw = new TextTableWindow(dir+filename,600,300);
//		mw = new TextWindow(path,300,500);

		TextTablePanel tp = mw.getTextTablePanel();
//		TextPanel tp = mw.getTextPanel();
		int lines = tp.getLineCount();
		String[] data = new String[lines];
		for (int i=0; i<lines; i++) {
			data[i] = tp.getLine(i);
		}
		mw.getTextPanel().setVisible(false);
		tp.setColumnHeadings(data[0]);
		for (int i=1; i<lines; i++) {
			tp.appendLine(data[i]);
		}
		Prefs.set("MFDB.muscletypelist",path);
		
	}
	
	public TextTableWindow openRabbitWindow() {
//	public TextWindow openRabbitWindow() {
		if (sw != null) return sw;
		return new TextTableWindow("Sample Types",rabCols,"",400,200);
//		return new TextWindow("Sample Types",rabCols,"",400,200);
	}
	
	public void addMuscle() {
		
		// Determine selected rabbit
		int rabbitIndex = sw.getTextTablePanel().getSelectionStart();
//		int rabbitIndex = sw.getTextPanel().getSelectionStart();

		// Determine selected muscle type
		mw = openMuscleWindow();
		int muscleIndex = mw.getTextTablePanel().getSelectionStart();
//		int muscleIndex = mw.getTextPanel().getSelectionStart();

		// Valid set of selections to add a muscle
		if (rabbitIndex != -1 || muscleIndex != -1) {

			// Find instance rabbit object which is selected
			String sR = sw.getTextTablePanel().getLine(rabbitIndex);
//			String sR = sw.getTextPanel().getLine(rabbitIndex);
			int sRInd = -1;
			for (int i=0; i<this.rabbitCount; i++) {
				if (sR.equals(this.rabbits[i].name)) sRInd = i;
			}
			
			// Find muscle type which is selected
			String sMus = mw.getTextTablePanel().getLine(muscleIndex);
//			String sMus = mw.getTextPanel().getLine(muscleIndex);
			int sMInd = -1;
			for (int i=0; i<this.muscleTypeCount; i++) {
				if (sMus.equals(this.muscleTypes[i].name)) sMInd = i;
			}
			
			// Make new muscle
			rabbits[sRInd].muscles[rabbits[sRInd].mCount] = new Muscle(this.muscleTypes[sMInd],rabbits[sRInd],"desc","date");
			
			// Increase muscle count
			rabbits[sRInd].mCount++;
		}
	}
	
	public TextTableWindow openMuscleWindow() {
//	public TextWindow openMuscleWindow() {
		if (mw != null) return mw;
		return new TextTableWindow("Muscle Samples","",300,500);
//		return new TextWindow("Muscle Samples","",300,500);
	}
	
	public void addMuscleType() {
		GenericDialog gd = new GenericDialog("Add Muscle Type");
		String newMTName = "";
		gd.addStringField("Name",newMTName,20);
		String newMTLocation = "location";
		gd.addStringField("Location",newMTLocation,20);
		String newMTType = "";
		gd.addStringField("Type",newMTType,20);
		String newPSlow = "";
		gd.addStringField("Percent Slow",newPSlow,20);
		String newNebKDa = "";
		gd.addStringField("Nebulin Size (kDa)",newNebKDa,20);
		String newNebErr = "";
		gd.addStringField("Nebulin Size Error",newNebErr,20);
		String newTitKDa = "";
		gd.addStringField("Titin Size (kDa)",newTitKDa,20);
		String newTitErr = "";
		gd.addStringField("Titin Size Error",newTitErr,20);
		String newTFL = "";
		gd.addStringField("Reported Thin Filament Length",newTFL,20);
		gd.showDialog();
		newMTName = gd.getNextString();
		newMTLocation = gd.getNextString();
		newMTType = gd.getNextString();
		newPSlow = gd.getNextString();
		newNebKDa = gd.getNextString();
		newNebErr = gd.getNextString();
		newTitKDa = gd.getNextString();
		newTitErr = gd.getNextString();
		newTFL = gd.getNextString();
		muscleTypes[muscleTypeCount] = new MuscleType(newMTName, newMTLocation, newMTType, 
				newPSlow, newNebKDa, newNebErr, newTitKDa, newTitErr, newTFL);
		mw = openMuscleTypeWindow();
		mw.getTextPanel().setVisible(false);
		mw.append(muscleTypes[muscleTypeCount].tabOut());
		this.rabbitCount++;

	}
	
	public TextTableWindow openMuscleTypeWindow() {
//	public TextWindow openMuscleTypeWindow() {
		if (mw != null) return mw;
		return new TextTableWindow("Muscle Types",muscleTypeCols,"",300,500);
//		return new TextWindow("Muscle Types",muscleTypeCols,"",300,500);
	}
	
	public void itemStateChanged(ActionEvent a) {}
	
	public void actionPerformed(ActionEvent e) {
		String label = e.getActionCommand();
		if (label==null)
			return;
		String command = label;
		if (command.equals("Image")) {
			OpenDialog od = new OpenDialog("Select Image","");
			imageName = od.getFileName();
		}
		
		System.out.println(imageName);
		tImage.setText(imageName);
//		imagePanel.repaint();
	}
	
	public void itemStateChanged(ItemEvent a) {}
}
/*	Muscle m0 = new Muscle(Rabbit.ps, rabbit1, "", "053006");
Muscle m1 = new Muscle(Rabbit.ps, rabbit1, "", "062906");
Muscle m2 = new Muscle(Rabbit.ps, rabbit1, "", "");
Muscle m3 = new Muscle(Rabbit.ps, rabbit1, "", "");
Muscle m4 = new Muscle(Rabbit.sol, rabbit1, "", "053006");
Muscle m5 = new Muscle(Rabbit.sol, rabbit1, "", "060806");
Muscle m6 = new Muscle(Rabbit.sol, rabbit1, "", "");
Muscle m7 = new Muscle(Rabbit.sol, rabbit1, "", "");
Muscle m8 = new Muscle(Rabbit.gast, rabbit1, "", "060806");
Muscle m9 = new Muscle(Rabbit.gast, rabbit1, "", "");
Muscle m10 = new Muscle(Rabbit.ta, rabbit1, "", "060806");
Muscle m11 = new Muscle(Rabbit.ta, rabbit1, "", "");
Muscle m12 = new Muscle(Rabbit.dia, rabbit1, "", "060806");
Muscle m13 = new Muscle(Rabbit.edl, rabbit1, "", "060806");
Muscle m14 = new Muscle(Rabbit.edl, rabbit1, "", "");
Muscle m15 = new Muscle(Rabbit.pd, rabbit1, "", "060806");
Muscle m16 = new Muscle(Rabbit.pd, rabbit1, "", "");
Muscle m17 = new Muscle(Rabbit.pap, rabbit1, "", "062906");
Muscle m18 = new Muscle(Rabbit.grac, rabbit1, "", "062906");
Muscle m19 = new Muscle(Rabbit.rf, rabbit1, "", "062906");
Muscle m20 = new Muscle(Rabbit.st, rabbit1, "", "062906");
Muscle m21 = new Muscle(Rabbit.dia, rabbit1, "", "062906");

Muscle m22 = new Muscle(Rabbit.ps, rabbit2, "", "071006");
Muscle m23 = new Muscle(Rabbit.ps, rabbit2, "", "");
Muscle m24 = new Muscle(Rabbit.sol, rabbit2, "", "071006");
Muscle m25 = new Muscle(Rabbit.sol, rabbit2, "", "");
Muscle m26 = new Muscle(Rabbit.gast, rabbit2, "", "071006");
Muscle m27 = new Muscle(Rabbit.gast, rabbit2, "", "");
Muscle m28 = new Muscle(Rabbit.ta, rabbit2, "", "071006");
Muscle m29 = new Muscle(Rabbit.ta, rabbit2, "", "");
Muscle m30 = new Muscle(Rabbit.dia, rabbit2, "", "071006");
Muscle m31 = new Muscle(Rabbit.dia, rabbit2, "", "");
Muscle m32 = new Muscle(Rabbit.edl, rabbit2, "", "071006");
Muscle m33 = new Muscle(Rabbit.edl, rabbit2, "", "");
Muscle m34 = new Muscle(Rabbit.pd, rabbit2, "", "071006");
Muscle m35 = new Muscle(Rabbit.pap, rabbit2, "", "071006");
Muscle m36 = new Muscle(Rabbit.grac, rabbit2, "", "071006");
Muscle m37 = new Muscle(Rabbit.grac, rabbit2, "", "");
Muscle m38 = new Muscle(Rabbit.tfl, rabbit2, "", "071006");
Muscle m39 = new Muscle(Rabbit.tfl, rabbit2, "", "");
Muscle m40 = new Muscle(Rabbit.ab, rabbit2, "", "071006");
Muscle m41 = new Muscle(Rabbit.ab, rabbit2, "", "");
Muscle m42 = new Muscle(Rabbit.bb, rabbit2, "", "071006");
Muscle m43 = new Muscle(Rabbit.bb, rabbit2, "", "");
Muscle m44 = new Muscle(Rabbit.cd, rabbit2, "", "071006");
Muscle m45 = new Muscle(Rabbit.cd, rabbit2, "", "");
Muscle m46 = new Muscle(Rabbit.fcu, rabbit2, "", "071006");
Muscle m47 = new Muscle(Rabbit.fdp, rabbit2, "", "071006");
Muscle m48 = new Muscle(Rabbit.rf, rabbit2, "", "071006");
Muscle m49 = new Muscle(Rabbit.rf, rabbit2, "", "");
Muscle m50 = new Muscle(Rabbit.sm, rabbit2, "", "071006");
Muscle m51 = new Muscle(Rabbit.sm, rabbit2, "", "");
Muscle m52 = new Muscle(Rabbit.st, rabbit2, "", "071006");
Muscle m53 = new Muscle(Rabbit.st, rabbit2, "", "");
Muscle m54 = new Muscle(Rabbit.pect, rabbit2, "", "071006");

Muscle m55 = new Muscle(Rabbit.ps, rabbit3, "", "");
Muscle m56 = new Muscle(Rabbit.sol, rabbit3, "", "");
Muscle m57 = new Muscle(Rabbit.sol, rabbit3, "", "");
Muscle m58 = new Muscle(Rabbit.grac, rabbit3, "", "");
Muscle m59 = new Muscle(Rabbit.grac, rabbit3, "", "");
Muscle m60 = new Muscle(Rabbit.ta, rabbit3, "", "");
Muscle m61 = new Muscle(Rabbit.gast, rabbit3, "", "");
Muscle m62 = new Muscle(Rabbit.gast, rabbit3, "", "");
Muscle m63 = new Muscle(Rabbit.dia, rabbit3, "", "");
Muscle m64 = new Muscle(Rabbit.fdp, rabbit3, "", "");
Muscle m65 = new Muscle(Rabbit.edl, rabbit3, "", "");
Muscle m66 = new Muscle(Rabbit.edl, rabbit3, "", "");

Muscle m67 = new Muscle(Rabbit.ps, rabbit4, "", "082106");
Muscle m68 = new Muscle(Rabbit.sol, rabbit4, "", "082106");
Muscle m69 = new Muscle(Rabbit.fdp, rabbit4, "", "");
Muscle m70 = new Muscle(Rabbit.tfl, rabbit4, "", "");
Muscle m71 = new Muscle(Rabbit.tfl, rabbit4, "", "");
Muscle m72 = new Muscle(Rabbit.cd, rabbit4, "", "");
Muscle m73 = new Muscle(Rabbit.cd, rabbit4, "", "");
Muscle m74 = new Muscle(Rabbit.ld, rabbit4, "", "");
Muscle m75 = new Muscle(Rabbit.ld, rabbit4, "", "");
Muscle m76 = new Muscle(Rabbit.pd, rabbit4, "", "");
Muscle m77 = new Muscle(Rabbit.pd, rabbit4, "", "");
Muscle m78 = new Muscle(Rabbit.dia, rabbit4, "", "");
Muscle m79 = new Muscle(Rabbit.dia, rabbit4, "", "");
Muscle m80 = new Muscle(Rabbit.at, rabbit4, "", "");
Muscle m81 = new Muscle(Rabbit.at, rabbit4, "", "");
Muscle m82 = new Muscle(Rabbit.bb, rabbit4, "", "");
*/	
