/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.Label;
import java.awt.Rectangle;
import java.awt.TextField;

public class LField extends TextField {
	public static final int EAST=0, NORTH=1, WEST=2, SOUTH=3;
	Label label;
	TextField field;
	Rectangle lb;
	Rectangle fb;
	private double fraction = 0.5;
	int direction;
	
	LField (String title) {
		super();
		label = new Label(title);
		direction = NORTH;
	}
	
	LField (String title, int labelLocation) {
		super();
		label = new Label(title);
		direction = (labelLocation % 4);
	}
	
	/**
	 * This method refers to the size of both the label and the text
	 * field.
	 * @param f is the fractional size of the TextField
	 */
	public void setBounds(int x, int y, int w, int h, double f) {
		if (f<0 || f>1) f = fraction;
		switch (direction) {
		case EAST: 
			fb = new Rectangle(x,y,(int)(w*f),h);
			lb = new Rectangle(x+(int)(w*f),y,(int)(w*(1-f)),h);
			break;
		case NORTH:	
			fb = new Rectangle(x,y+(int)(h*(1-f)),w,(int)(h*f));
			lb = new Rectangle(x,y,w,(int)(h*(1-f)));
			break;
		case WEST:
			fb = new Rectangle(x+(int)(w*(1-f)),y,(int)(w*f),h);
			lb = new Rectangle(x,y,(int)(w*(1-f)),h);
			break;
		case SOUTH:
			fb = new Rectangle(x,y,w,(int)(h*f));
			lb = new Rectangle(x,y+(int)(h*f),w,(int)(h*(1-f)));
			break;
		}
		super.setBounds(fb);
		label.setBounds(lb);
	}
}

