/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

import ij.IJ;
import ij.ImagePlus;
import ij.Prefs;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.gui.Line;
import ij.io.DirectoryChooser;
import ij.io.FileInfo;
import ij.io.FileSaver;
import ij.io.OpenDialog;
import ij.io.Opener;
import ij.io.SaveDialog;
import ij.text.TextPanel;
import ij.text.TextWindow;
import ij.util.Tools;

public class LSDDecon_Loader {

	public static final char FIXED = LSDDecon_Loader.FIXED;
	public static final char FREE = LSDDecon_Loader.FREE;
	public static final char SEP_CHAR = LSDDecon_Saver.SEP_CHAR;
	
	public static final int CAT_IMG_DATA = 0;
	public static final int LINE = 1;
	public static final int LS_DATA = 2;
	public static final int RM_DATA = 3;
	public static final int SMODEL_DATA = 4;
	public static final int PSF_DATA = 5;	
	public static final int FIT_DATA = 6;
	public static final int ALL = 7;
	String baseDirectory = IJ.getDirectory("home");
	
	
	String resultsSavePath = "";//IJ.getDirectory("startup");
	
	LSDDecon_Manager manager;
	CatalogedImageInfo catImageInfo;
	TextWindow results = null;
	LSprocessor currLS;
	DDwindow ddwin;
	LSwindow lswin;
	RepeatModel currRM;
	DDplotter ddp;
	DDCurveFitter fitter;
	
	String[] values = null;

	double version;
	String impFileName;
	int impSlice;
	String lineScanID;
	Line lsLine;
	double skewAngle;
	String subImageFileName;
	int lsWidth;
	int lsLength;
	double lsAngle;
	int[] lsWidths;
	int[] lsModes;
	int bgCorrIndex;
	String bgCorrStr;
	String lsInfoStr;
	Point2D.Double[] repts;
	String smodelTitle;
	double[] smodelParams;
	String psfTitle;
	String psfDesc;
	double psfFocusParam;
	String ddInfoStr;
	String fitMethod;
	double error;
	double goodness;
	double correl;
	double perArea;
	double dataParamRatio;
	int numFreeParams;
	int numTotalParams;
	double tolerance;
	int numIter;
	int maxIter;
	int numRestarts;
	String freeParamsStr;
	double intSrchR;
	double posSrchR;
	int maxOverhang;
	String fitInfoStr;
	
	
	
	public LSDDecon_Loader(LSDDecon_Manager manager) {
		this.manager = manager;
		this.currLS = manager.currLS;
		this.currRM = manager.currRM;
		this.ddwin = manager.ddwin;
		this.lswin = manager.lswin;
		this.ddp = manager.ddp;
		this.fitter = manager.fitter;
		this.results = manager.results;
	}
	
	public void loadCatImageData(String path) {
		
	}
	
	public void loadSubImage2(String path) {
		
	}
	
	public void loadLineScan() {
//	public LSprocessor loadLineScan() {
		OpenDialog od = new OpenDialog("Load LineScan from Results.txt file","");
		String dir = od.getDirectory();
		String fileName = od.getFileName();
		loadLineScan(dir+fileName);
//		return loadLineScan(dir+fileName);
	}
	
	public void loadLineScan(String path) {
//	public LSprocessor loadLineScan(String path) {
		loadResults(path,LS_DATA);
		Opener o = new Opener();
		String dir = o.getDir(path);
		String impPath = dir + this.impFileName;
//		System.out.println("Loader.loadLineScan: impPath="+impPath);
		Line line = this.lsLine;
		int slice = this.impSlice;
		int[] regWidths = this.lsWidths;
		int[] regModes = this.lsModes;
		String info = this.lsInfoStr;
		ImagePlus imp;
		if (impPath.endsWith(".dv")) {
			Object obj = IJ.runPlugIn("Deltavision_Opener",impPath);
			imp = IJ.getImage();
		}
		else {
			imp = o.openImage(impPath);
		}
		if (imp!=null) {
			imp.setSlice(impSlice);
			imp.setRoi(lsLine);
		}
		LineRegion lr = new LineRegion(line);
		lr.regWidths = this.lsWidths;
		lr.regModes = this.lsModes;
		lr.skewAngle = this.skewAngle;
		lr.bgCorrIndex = this.bgCorrIndex;
		String lineName = line.getName();
//		System.out.println("Loader.loadLineScan: lineName = "+lineName);
		manager.list.add(lineName);
		manager.rois.put(lineName,lr);
		manager.info.setText(manager.getInfoText(lineName));
		manager.list.select(manager.list.getItemCount());
//		LSprocessor ls = new LSprocessor(imp,lr);
//		manager.rois.put(line.getName(),line);
//		LSprocessor ls = new LSprocessor(imp,lsLine,impSlice,skewAngle,lsWidths,lsModes,bgCorrIndex,lsInfoStr);
//		return ls;
	}
	
	public void loadResults(String path, int subset) {
		IJ.showStatus("Opening: " + path);
		try {
			BufferedReader r = new BufferedReader(new FileReader(path));
			values = load(r);
			r.close();
		}
		catch (Exception e) {
			IJ.error(e.getMessage());
		}
		IJ.showStatus("");
		if (values != null && values.length > 0) {
			parseValues(values, subset);
		}
	}
	
	public String[] load(BufferedReader in) throws IOException {
		String header = in.readLine();
		String data = in.readLine();
		String[] values = Tools.split(data,""+SEP_CHAR);
		return values;
	}
	
	public void parseValues(String[] values, int subset) {
		int index = 0;
//		System.out.println("Loader.paseValues: values["+index+"]= "+values[index]);
		// For reading files which the version was not written
		String v = values[index];
		try {
			version = Double.parseDouble(v);
			index++;
			impFileName = values[index++];
		} catch (NumberFormatException e) {
			System.out.println("Exception "+ e.toString() + " caught");
			impFileName = v;
			index++;
		}
		

//		version = Double.parseDouble(values[index++]);
//		impFileName = values[index++];
		impSlice = Integer.parseInt(values[index++]);
//		System.out.println("Loader.paseValues: impSlice= "+impSlice);
		
		if (subset>=LINE) {
			String lineScanID = values[index++];
			double x1d = Double.parseDouble(values[index++]);
			double y1d = Double.parseDouble(values[index++]);
			double x2d = Double.parseDouble(values[index++]);
			double y2d = Double.parseDouble(values[index++]);
			lsLine = new Line((int)x1d,(int)y1d,(int)x2d,(int)y2d);
			lsLine.setName(lineScanID);
			lsLine.x1d = x1d;
			lsLine.y1d = y1d;
			lsLine.x2d = x2d;
			lsLine.y2d = y2d;
		}
		
		if (subset>=LS_DATA) {
			skewAngle = Double.parseDouble(values[index++]);
			subImageFileName = values[index++];
			lsWidth = Integer.parseInt(values[index++]);
			lsLength = Integer.parseInt(values[index++]);
			lsAngle = Double.parseDouble(values[index++]);
			int numRegions = Integer.parseInt(values[index++]);
			lsWidths = new int[numRegions];
			for (int i=0; i<numRegions; i++) {
				lsWidths[i] = Integer.parseInt(values[index++]);
			}
			lsModes = new int[numRegions];
			for (int i=0; i<numRegions; i++) {
				lsModes[i] = Integer.parseInt(values[index++]);
			}
			bgCorrIndex = Integer.parseInt(values[index++]);
			bgCorrStr = values[index++];
			lsInfoStr = values[index++];
		}
		
		if (subset>=RM_DATA) {
			int numRepts = Integer.parseInt(values[index++]);
			repts = new Point2D.Double[numRepts];
			for (int i=0; i<numRepts; i++) {
				repts[i].x = Double.parseDouble(values[index++]);
			}
			for (int i=0; i<numRepts; i++) {
				repts[i].y = Double.parseDouble(values[index++]);
			}
		}
		
		if (subset>=SMODEL_DATA) {
			smodelTitle = values[index++];
			int numSModelParams = Integer.parseInt(values[index++]);
			smodelParams = new double[numSModelParams];
			for (int i=0; i<numSModelParams; i++) {
				smodelParams[i] = Double.parseDouble(values[index++]);
			}
		}
		
		if (subset>=PSF_DATA) {
			psfTitle = values[index++];
			psfDesc = values[index++];
			psfFocusParam = Double.parseDouble(values[index++]);
			ddInfoStr = values[index++];
		}
		
		if (subset>=FIT_DATA) {
			fitMethod = values[index++];
			error = Double.parseDouble(values[index++]);
			goodness = Double.parseDouble(values[index++]);
			correl = Double.parseDouble(values[index++]);
			perArea = Double.parseDouble(values[index++]);
			dataParamRatio = Double.parseDouble(values[index++]);
			numFreeParams = Integer.parseInt(values[index++]);
			numTotalParams = Integer.parseInt(values[index++]);
			tolerance = Double.parseDouble(values[index++]);
			numIter = Integer.parseInt(values[index++]);
			maxIter = Integer.parseInt(values[index++]);
			numRestarts = Integer.parseInt(values[index++]);
			freeParamsStr = values[index++];
			intSrchR = Double.parseDouble(values[index++]);
			posSrchR = Double.parseDouble(values[index++]);
			maxOverhang = Integer.parseInt(values[index++]);
			fitInfoStr = values[index++];
		}
	}
	
	/** If a Distributed Deconvolution Results Window is already open, this gets it, otherwise it opens it 
	 * as a text file.
	 * Uses a routine to rebuild the heading from the first line.
	 * Called by:
	 * 		LSDDecon_Manager.actionPerformed(ActionEvent)
	 * 
	 */
	public TextWindow openResults(String path) {
		if (results != null) return results;
		
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		int h = 200;
		String name = null;
		OpenDialog od = new OpenDialog("Open Results Text File...",path);
		String directory = od.getDirectory();
		name = od.getFileName();
		if (name!=null) {
			path = directory + name;
			results = new TextWindow(path,d.width,h);
			if (results!=null) {
				reloadResults();
	    		results.setLocation(0,d.height-h-30);
	    		results.addWindowListener(manager);
				return results;
			}
		}
		String resultsName = "Distributed Deconvolution Results";
		results = new TextWindow(resultsName,"",d.width,h);
		results.getTextPanel().setColumnHeadings(" " + getLabelHeadingString());
		results.setLocation(0,d.height-h-30);
		results.addWindowListener(manager);
		return results;
	}
	
	public void reloadResults() {
		TextPanel panel = results.getTextPanel();
		int lines = panel.getLineCount();
		String headings;
		String[] data = {};
		if (lines>1) {
			data = new String[lines-1];
			for (int i=0; i<lines-1; i++) {
				data[i] = panel.getLine(i+1);
			}
		}
		if (lines>0) {
			headings = panel.getLine(0);
			panel.setColumnHeadings(headings); // resets text
		}
		for (int i=0; i<data.length; i++) {
			panel.appendLine(data[i]);
		}
	}
	
	/** Loads a linescan from a Distributed Deconvolution Results Window.
	 * Makes a new LSinfo object from the data.
	 * Called by:
	 * 		LSDDecon_Manager.actionPerformed(ActionEvent)
	 * 
	 */
	public void loadResultsLine(int index) {
		// Make sure results window is open and set
		if (results == null) {
			Frame win = WindowManager.getFrame("Distributed Deconvolution Results");
			if (win!=null){
				// set an open results window from a previous analysis.
				results = (TextWindow)win;
			}
			else {
				// open up lastUsedResults (XXX currently set to null, call Prefs.set somewhere)
				results = openResults(resultsSavePath);
				
				// Do nothing if canceled or file does not open
				if (results==null) return;
			}
		}

		if (index == 0) {
			// Load line dialog
			GenericDialog gd = new GenericDialog("Load Results");
			gd.addNumericField("Record Number",0,0);
			gd.showDialog();
			index = (int)gd.getNextNumber()-1;
		}
		// Construct a new LSinfo object from the results and add it to the Line List.
	    	String text = results.getTextPanel().getLine(index);
	    	FileInfo ofi = new FileInfo();
	    	String[] fields = Tools.split(text,"\t");
	    	int nFields = fields.length;
	    	
	    	for (int i=0; i<nFields; i++) {
//	    		System.out.println("LSinfo(String).<init>: field["+i+"]= "+fields[i]);
	    		IJ.write("LSinfo(String).<init>: field["+i+"]= "+fields[i]);
	    	}

	    	String linescanID = "";
	    	Line line = null;
	    	if (nFields>=83) {
	    		ofi.fileName = fields[1];
	    		linescanID = fields[2];
	        	int index2 = 69;//70;
	    		int lsLength = Integer.parseInt(fields[index2++]);
	    		int lsWidth = Integer.parseInt(fields[index2++]);
	    		double lsAngle = Double.parseDouble(fields[index2++])*Math.PI/180;
	    		int bg1w = Integer.parseInt(fields[index2++]);
	    		int rawLSw = Integer.parseInt(fields[index2++]);
	    		int bg2w = Integer.parseInt(fields[index2++]);
	    		int[] regionCalcs = new int [3];
	    		for (int i=0; i<3; i++) {
	    			String calcMode = fields[index2++];
	    			if (calcMode.equalsIgnoreCase("Minimum")) regionCalcs[i] = 1;
	    			if (calcMode.equalsIgnoreCase("Average")) regionCalcs[i] = 2;
	    			if (calcMode.equalsIgnoreCase("Median")) regionCalcs[i] = 4;
	    			if (calcMode.equalsIgnoreCase("Maximum")) regionCalcs[i] = 8;
	    		}
	    		int bg1calc = regionCalcs[0];
	    		int rawLScalc = regionCalcs[1];
	    		int bg2calc = regionCalcs[2];
	    		String corrcalcStr = fields[index2++];
	    		int corrcalc = 2;
//	    		corrcalc = 2; //XXX Prefs.set(corrcalc)?
	    		if (corrcalcStr!=null) {
		    		for (int i=0; i<LSprocessor.BG_MODES.length; i++) {
		    			if (corrcalcStr.equals(LSprocessor.BG_MODES[i])) corrcalc = i;
		    		}
	    		}
	    		else {
	    			corrcalcStr.equals(LSprocessor.BG_MODES[corrcalc]);
	    		}
	    		double x1d = Double.parseDouble(fields[index2++]);
	    		double y1d = Double.parseDouble(fields[index2++]);
	    		double x2d = Double.parseDouble(fields[index2++]);
	    		double y2d = Double.parseDouble(fields[index2++]);
	        	line = new Line((int)x1d,(int)y1d,(int)x2d,(int)y2d);
	    		line.x1d = x1d;
	    		line.y1d = y1d;
	    		line.x2d = x2d;
	    		line.y2d = y2d;
	    		ofi.fileName = fields[84];
	    		ofi.directory = fields[85];
	    		ofi.fileFormat = FileInfo.GRAY16_SIGNED; //fields[86]; XXX Update
	    		ofi.fileType = FileInfo.RAW; //fields[87]; XXX Update
	    		
	    		index2 = 87;//88;
	    		int slice = Integer.parseInt(fields[index2++]);
	    		ofi.nImages = Integer.parseInt(fields[index2++]);
	    		ofi.height = Integer.parseInt(fields[index2++]);
	    		ofi.width = Integer.parseInt(fields[index2++]);
	    		ofi.pixelDepth = Double.parseDouble(fields[index2++]);
	    		ofi.pixelHeight = Double.parseDouble(fields[index2++]);
	    		ofi.pixelWidth = Double.parseDouble(fields[index2++]);
	    		ofi.description = fields[index2++];
	    		int n = ofi.description.indexOf("LSDDecon v");
	    		String s = "";
				if (n>0) s = ofi.description.substring(n+10,n+14);
//				System.out.println("LSinfo.<init>: s= "+s);
	    		double version;
	    			try {
	    				version = Double.parseDouble(s);
	    			}
	    			catch (NumberFormatException e) {
	    				version = 0;
	    			}
	    			
	    		// Additional fields to add here
	    		double skewAngle = 0;
	    		if (version>=3) {
	    			skewAngle = Double.parseDouble(fields[index2++]);
	    		}
	    		
	    		String info = "";
	    		for (int i=index2; i<nFields; i++) {
	        		info = info + "; " + fields[index2++];
	    		}
	    		
	/*    		String fitInfo = fields[index++];
	    		String ddeconInfo = fields[index++];
	    		this.infoLS = fields[index++];
	    		fi.info = fields[index];
	*/    		
	    		
	    	}
//		LSinfo lsi = new LSinfo(results.getTextPanel().getLine(index));
//		IJ.write("Manager.loadResultsLine: lsi="+lsi.toString());
//		lsi.manager = this;
		String label = getUniqueLSName(linescanID);
//		String label = this.getUniqueLSName(lsi.linescanID);
		manager.list.add(label);
		manager.rois.put(label,line);
		manager.info.setText(manager.getInfoText(label));
		manager.list.select(manager.list.getItemCount());
	}
	
	/** 
	 * Called by:
	 * 		LSDDecon_Manager.getInfo(String)
	 * 		LSDDecon_Manager.loadResultsLine()
	 * 
	 * @param name
	 */
	public String getUniqueLSName(String name) {
			String name2 = name;
			int n = 1;
			Line roi2 = (Line)manager.rois.get(name2);
//			LSinfo roi2 = (LSinfo)rois.get(name2);
			while (roi2!=null) {
					int lastDash = name2.lastIndexOf("-");
					if (lastDash!=-1 && name2.length()-lastDash<5)
						name2 = name2.substring(0, lastDash);
					name2 = name2+"-"+n;
					n++;
					roi2 = (Line)manager.rois.get(name2);
//				roi2 = (LSinfo)rois.get(name2);
			}
			return name2;
	}
	
    /** Returns headings for Distributed Deconvolution Results Window
     * Called by:
     * 	DDinfo.summarize(Boolean)
     * 
     * @return tab-delimited string of heading names
     */
    public String getLabelHeadingString() {
    	int maxZ = RepeatModel.MAX_NUMBER_REPEATS;
    	String[] labels1 = {"Img FileName","Linescan ID","Date","Sample","Fluor","Exp. Time",
    			"Model","Error","GoodFit","Corr. Coeff.","Per. Area","Data Points","Z Lines"};
    	String[] Zlabels = new String [2*maxZ];
    	for (int i=0; i<maxZ; i++) {
    		Zlabels[i] = "Z" + (i+1) + " Pos\t";
    		Zlabels[i+maxZ] = "Z" + (i+1) + " Wght\t";
    	}
    	String[] labels2x;
    	if (Double.parseDouble(LSDDecon_Manager.VERSION)>=3) {
    		String[] labels2 = {"Fit Method","Data/FreeParams","Free Params","Total Params",
    			"Tolerance","Num. Iter.","Max. Iter","Restarts","Params Fixed",
    			"Int. Search Rad.","Pos. Search Rad.","Max. Overhang",
    			"LS Length","LS Width(Total)","LS angle",
    			"BG1 Width","rawLS Width","BG2 Width",
    			"BG1 Calc","rawLS Calc","BG2 Calc","LS correction",
    			"ROI X1","ROI X2","ROI Y1","ROI Y2",
    			"Img FileName","Img Directory","Img Format","Img Type",
    			"Img Frame","Img Stack Size","Img Height","Img Width",
    			"Img Pixel Depth","Img Pixel Height","Img Pixel Width",
    			"Description","SkewAngle","Fit Info","DDecon Info","LS Info","Img Info"};
    		labels2x=labels2;
    	}
    	else {
    		String[] labels2 = {"Fit Method","Data/FreeParams","Free Params","Total Params",
    			"Tolerance","Num. Iter.","Max. Iter","Restarts","Params Fixed",
    			"Int. Search Rad.","Pos. Search Rad.","Max. Overhang",
    			"LS Length","LS Width(Total)","LS angle",
    			"BG1 Width","rawLS Width","BG2 Width",
    			"BG1 Calc","rawLS Calc","BG2 Calc","LS correction",
    			"ROI X1","ROI X2","ROI Y1","ROI Y2",
    			"Img FileName","Img Directory","Img Format","Img Type",
    			"Img Frame","Img Stack Size","Img Height","Img Width",
    			"Img Pixel Depth","Img Pixel Height","Img Pixel Width",
    			"Description","Fit Info","DDecon Info","LS Info","Img Info"};
    		labels2x=labels2;
    	}
    	StringBuffer output = new StringBuffer(200);
    	output.append("\t");
    	for (int i=0; i<labels1.length; i++) {
    		output.append(labels1[i] + "\t");
    	}
    	for (int j=0; j<currRM.model.getParamNames().length; j++) {
    		output.append(currRM.model.getParamNames()[j] + "\t");
    	}
    	for (int j=0; j<2*maxZ; j++) {
    		output.append(Zlabels[j]);
    	}
    	for (int i=0; i<labels2x.length; i++) {
    		output.append(labels2x[i] + "\t");
    	}
    	return new String(output);
    }
    
    public void loadSubImage(String name) {
    	String lsIDprefix = manager.prefixField.getText();
		String linescanID = getUniqueLSName(lsIDprefix+"-1"); // Assign new linescanID
    	String basename = currLS.imp.getShortTitle()+"_"+currLS.slice+"_"+linescanID;
    	String defName = currLS.imp.getOriginalFileInfo().fileName + "_" + currLS.slice + "_" + linescanID + ".tif";
    	String dir = currLS.imp.getOriginalFileInfo().directory;
    	if (name.equals("")) {
    		OpenDialog od = new OpenDialog("Open TIFF subimage",dir,defName);
    		dir = od.getDirectory();
    		name = od.getFileName();
    	}
    	Opener o = new Opener();
    	currLS.subImage = o.openTiff(dir,name);
    }
    
}
