/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import ij.IJ;
import ij.ImagePlus;
import ij.Prefs;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.gui.Line;
import ij.io.FileInfo;
import ij.io.FileSaver;
import ij.io.OpenDialog;
import ij.io.Opener;
import ij.io.SaveDialog;
import ij.text.TextPanel;
import ij.text.TextWindow;
import ij.util.Tools;

public class LSDDecon_Saver {

	public static final char FIXED = '1', FREE = '0';
	public static final char SEP_CHAR = '\t';
	String baseDirectory = IJ.getDirectory("home");
	
	String catalogPath = IJ.getDirectory("startup");
	String subImageSavePath = IJ.getDirectory("startup");
	String intensitySavePath = IJ.getDirectory("startup");
	String resultsSavePath = "";//IJ.getDirectory("startup");
	String subImageInfo = null;
	
	LSDDecon_Manager manager;
	CatalogedImageInfo catImageInfo;
	TextWindow results = null;
	LSprocessor currLS;
	DDwindow ddwin;
	LSwindow lswin;
	RepeatModel currRM;
	DDplotter ddp;
	DDCurveFitter fitter;
	
	
	
	public LSDDecon_Saver(LSDDecon_Manager manager) {
		this.manager = manager;
		this.currLS = manager.currLS;
		this.currRM = manager.currRM;
		this.ddwin = manager.ddwin;
		this.lswin = manager.lswin;
		this.ddp = manager.ddp;
		this.fitter = manager.fitter;
		this.results = manager.results;
	}
	
	public void exportAll() {
		String dir = currLS.imp.getOriginalFileInfo().directory;
		saveSubImage(dir);
//		saveLineDataFile(dir + currLS.lsName + ".txt");
//		saveLSIntsFile(dir + currLS.lsName + "_Ints.txt");
//		saveFittingResult(dir + currLS.lsName + "_FitResults.txt");
		saveSarcView(dir + currLS.lsName + "_Results.txt");
	}
	
	public void exportCatalogedData() {
		catImageInfo.save();
	}
	
	public void exportLSdata() {
		String dir = currLS.imp.getOriginalFileInfo().directory;
		saveSubImage(dir);
		saveLineDataFile(dir + currLS.lsName + ".txt");
		saveLSIntsFile(dir + currLS.lsName + "_Ints.txt");
	}
	
	public void exportFittingResult() {
		String dir = currLS.imp.getOriginalFileInfo().directory;
		saveFittingResult(dir + currLS.lsName + "_FitResults.txt");
		saveSarcView(dir + currLS.lsName + "_SarcView.txt");
	}
	
    public void saveSubImage(String dir) {
    	IJ.showStatus("Saving subImage to "+ dir);
    	setSubImageInfo(dir);
    	FileSaver fs = new FileSaver(currLS.subImage);
    	fs.save();
//    	IJ.showStatus("");
    }
    
    public void setSubImageInfo(String dir) {
    	// May want to calibrate this - is it already????
    	// calibration.info may be used to save this fi.info data
    	FileInfo fi = currLS.subImage.getFileInfo();
    	fi.directory = dir;
    	fi.fileName = currLS.subImage.getTitle();
    	fi.fileFormat = FileInfo.TIFF;
    	fi.description = LSDDecon_Manager.DESCRIPTION;
    	fi.info = "skewAngle=" + currLS.skewAngle
    			+ "; regionWidths=[" + currLS.regions[0].rect.height
    			+ "," + currLS.regions[1].rect.height
    			+ "," + currLS.regions[2].rect.height
    			+ "]"
    			+ "; line: x1d,y1d=(" + IJ.d2s(currLS.linePts[0].x,2)
    			+ "," + IJ.d2s(currLS.linePts[0].y,2)
    			+ "); x2d,y2d=(" + IJ.d2s(currLS.linePts[1].x,2)
    			+ "," + IJ.d2s(currLS.linePts[1].y,2)
//    			+ "; line: x1d,y1d=(" + IJ.d2s(currLS.line.x1d,2)
//    			+ "," + IJ.d2s(currLS.line.y1d,2)
//    			+ "); x2d,y2d=(" + IJ.d2s(currLS.line.x2d,2)
//    			+ "," + IJ.d2s(currLS.line.y2d,2)
    			+ ")";
    	currLS.subImage.setFileInfo(fi);
    }
    
	public void saveLineDataFile(String path) {
		PrintWriter pw = null;
		try {
			FileOutputStream fos = new FileOutputStream(path);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			pw = new PrintWriter(bos);
		}
		catch (IOException e) {
			//IJ.write("" + e);
			return;
		}
		pw.println(getLSLabels());
		pw.println(getLSData());
		pw.close();
		
	}
	
	public void saveLSIntsFile(String path) {
		PrintWriter pw = null;
		try {
			FileOutputStream fos = new FileOutputStream(path);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			pw = new PrintWriter(bos);
		}
		catch (IOException e) {
			//IJ.write("" + e);
			return;
		}
		pw.println(getLSLabels());
		pw.println(getLSData());
		pw.println();
		
		pw.println(getLSIntsLabel());
    	for (int i=0; i<currLS.lsInts.length; i++) {
    		pw.println(this.getLSInts(i));
    	}
    	pw.close();
	}
	
	public void saveFittingResult(String path) {
		PrintWriter pw = null;
		try {
			FileOutputStream fos = new FileOutputStream(path);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			pw = new PrintWriter(bos);
		}
		catch (IOException e) {
			//IJ.write("" + e);
			return;
		}
		pw.println(getFitLabels());
		pw.println(getFitData());
		pw.println();
		
		pw.println(getDDIntsLabel());
		for (int i=0; i<currRM.ddLength; i++) {
			pw.println(getDDInts(i));
		}
		pw.close();
	}
	
	public void saveSarcView(String path) {
		IJ.showStatus("Saving data to "+ path);
		PrintWriter pw = null;
		try {
			FileOutputStream fos = new FileOutputStream(path);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			pw = new PrintWriter(bos);
		}
		catch (IOException e) {
			//IJ.write("" + e);
			return;
		}
		pw.println(getFitLabels());
		pw.println(getFitData());
		pw.println();
		
		pw.println(getSarcViewLabel());
		for (int i=0; i<currRM.ddLength; i++) {
			pw.println(getSarcViewInts(i));
		}
		pw.close();
//		IJ.showStatus("");
	}
	
	public String getCatImageLabels() {
		StringBuffer sb = new StringBuffer(200);
		sb.append("Version" + SEP_CHAR);
		sb.append("Img FileName" + SEP_CHAR);
		sb.append("Img Directory" + SEP_CHAR);
		sb.append("Experiment" + SEP_CHAR);
		sb.append("Sample" + SEP_CHAR);
		sb.append("Muscle" + SEP_CHAR);
		sb.append("Probe" + SEP_CHAR);
		sb.append("Img Description" + SEP_CHAR);
		sb.append("Img Info" + SEP_CHAR);
		sb.append("Exposure Time" + SEP_CHAR);
		sb.append("Img Pixel Depth" + SEP_CHAR);
		sb.append("Img Pixel Height" + SEP_CHAR);
		sb.append("Img Pixel Width" + SEP_CHAR);
		return new String(sb);
	}
	
	public String getCatImageData() {
		StringBuffer sb = new StringBuffer(200);
		sb.append(LSDDecon_Manager.VERSION + SEP_CHAR);
		sb.append(catImageInfo.impName + SEP_CHAR);
		sb.append(catImageInfo.impDir + SEP_CHAR);
		sb.append(catImageInfo.experiment + SEP_CHAR);
		sb.append(catImageInfo.sample + SEP_CHAR);
		sb.append(catImageInfo.muscle + SEP_CHAR);
		sb.append(catImageInfo.probe + SEP_CHAR);
		sb.append(catImageInfo.description + SEP_CHAR);
		sb.append(catImageInfo.info + SEP_CHAR);
		sb.append(catImageInfo.timeExp + SEP_CHAR);
		sb.append(catImageInfo.pixelDepth + SEP_CHAR);
		sb.append(catImageInfo.pixelHeight + SEP_CHAR);
		sb.append(catImageInfo.pixelWidth + SEP_CHAR);
		return new String(sb);
	}
	
	public String getLSLabels() {
		StringBuffer sb = new StringBuffer();
		sb.append("Version" + SEP_CHAR);
		sb.append("ImageName" + SEP_CHAR);
		sb.append("Slice" + SEP_CHAR);
		sb.append("Linescan ID" + SEP_CHAR);
		sb.append("X1d" + SEP_CHAR);
		sb.append("Y1d" + SEP_CHAR);
		sb.append("X2d" + SEP_CHAR);
		sb.append("Y2d" + SEP_CHAR);
		sb.append("Skew" + SEP_CHAR);
		sb.append("SubImageName" + SEP_CHAR);
		sb.append("LineScan Width" + SEP_CHAR);
		sb.append("LineScan Length" + SEP_CHAR);
		sb.append("LineScan Angle" + SEP_CHAR);
		sb.append("Regions" + SEP_CHAR);
		for (int i=0; i<manager.currLS.regions.length; i++) {
			sb.append("R" + i + " width" + SEP_CHAR);
		}
		for (int i=0; i<manager.currLS.regions.length; i++) {
			sb.append("R" + i + " mode" + SEP_CHAR);
		}
		sb.append("BG Corr Index" + SEP_CHAR);
		sb.append("BG Corr Mode" + SEP_CHAR);
		sb.append("LineScan Info" + SEP_CHAR);
		sb.append("Repeats" + SEP_CHAR);
		return new String(sb);
	}
	
	public String getLSData() {
		StringBuffer sb = new StringBuffer();
		sb.append(LSDDecon_Manager.VERSION + SEP_CHAR);
		sb.append(currLS.imp.getOriginalFileInfo().fileName + SEP_CHAR);
		sb.append(IJ.d2s(currLS.slice,0) + SEP_CHAR);
		sb.append(currLS.lsName + SEP_CHAR);
		sb.append(IJ.d2s(currLS.linePts[0].x,2) + SEP_CHAR);
		sb.append(IJ.d2s(currLS.linePts[0].y,2) + SEP_CHAR);
		sb.append(IJ.d2s(currLS.linePts[1].x,2) + SEP_CHAR);
		sb.append(IJ.d2s(currLS.linePts[1].y,2) + SEP_CHAR);
//		sb.append(IJ.d2s(currLS.line.x1d,2) + SEP_CHAR);
//		sb.append(IJ.d2s(currLS.line.y1d,2) + SEP_CHAR);
//		sb.append(IJ.d2s(currLS.line.x2d,2) + SEP_CHAR);
//		sb.append(IJ.d2s(currLS.line.y2d,2) + SEP_CHAR);
		sb.append(IJ.d2s(currLS.skewAngle,2) + SEP_CHAR);
		sb.append(currLS.subImage.getOriginalFileInfo().fileName + SEP_CHAR);
		sb.append(IJ.d2s(currLS.lsWidth,0) + SEP_CHAR);
		sb.append(IJ.d2s(currLS.lsLength,0) + SEP_CHAR);
		sb.append(IJ.d2s(currLS.lsAngle,2) + SEP_CHAR);
		sb.append(IJ.d2s(manager.currLS.regions.length,0) + SEP_CHAR);
		for (int i=0; i<manager.currLS.regions.length; i++) {
			sb.append(IJ.d2s(manager.currLS.regions[i].rect.height,0) + SEP_CHAR);
		}
		for (int i=0; i<manager.currLS.regions.length; i++) {
			sb.append(IJ.d2s(manager.currLS.regions[i].mode,0) + SEP_CHAR);
		}
		sb.append(IJ.d2s(currLS.bgCorrIndex,0) + SEP_CHAR);
		sb.append(LSprocessor.BG_MODES[currLS.bgCorrIndex] + SEP_CHAR);
		sb.append(("" + currLS.lsInfo) + SEP_CHAR);
		
		int numRepts = 0;
		if (manager.currRM != null) numRepts = manager.currRM.repts.length;
		sb.append(IJ.d2s(numRepts,0) + SEP_CHAR);
		return new String(sb);
	}
	
	public String getFitLabels() {
		StringBuffer sb = new StringBuffer(getLSLabels());
    	for (int i=0; i<currRM.repts.length; i++) {
    		sb.append("Z" + i + " position" + SEP_CHAR);
    	}
    	for (int i=0; i<currRM.repts.length; i++) {
    		sb.append("Z" + i + " weight" + SEP_CHAR);
    	}
		sb.append("SModel" + SEP_CHAR);
		sb.append("#SModel Params" +SEP_CHAR);
		String[] paramNames = currRM.model.getParamNames();
    	for (int i=0; i<paramNames.length; i++) {
    		sb.append(paramNames[i] + SEP_CHAR);
    	}
    	sb.append("PSFModel" + SEP_CHAR);
    	sb.append("PSF Desc." + SEP_CHAR);
    	sb.append("Focus Param" + SEP_CHAR);
		sb.append("DDecon Info" + SEP_CHAR);

		// Fitting Results
		sb.append("Fit Method" + SEP_CHAR);
    	sb.append("Error" + SEP_CHAR);
		sb.append("GoodFit" + SEP_CHAR);
		sb.append("Corr. Coeff." + SEP_CHAR);
		sb.append("Per. Area" + SEP_CHAR);
		sb.append("Data/FreeParams" + SEP_CHAR);
		sb.append("Free Params" + SEP_CHAR);
		sb.append("Total Params" + SEP_CHAR);
		sb.append("Tolerance" + SEP_CHAR);
		sb.append("Num. Iter." + SEP_CHAR);
		sb.append("Max. Iter" + SEP_CHAR);
		sb.append("Restarts" + SEP_CHAR);
		sb.append("Params Fixed" + SEP_CHAR);
		sb.append("Int. Search Rad." + SEP_CHAR);
		sb.append("Pos. Search Rad." + SEP_CHAR);
		sb.append("Max. Overhang" + SEP_CHAR);
		sb.append("Fit Info" + SEP_CHAR);
    	return new String(sb);
	}
	
	public String getFitData() {
		StringBuffer sb = new StringBuffer(getLSData());
    	for (int i=0; i<currRM.repts.length; i++) {
    		sb.append(IJ.d2s(currRM.repts[i].x,3) + SEP_CHAR);
    	}
    	for (int i=0; i<currRM.repts.length; i++) {
    		sb.append(IJ.d2s(currRM.repts[i].y,3) + SEP_CHAR);
    	}
		sb.append(currRM.model.getTitle() + SEP_CHAR);
		sb.append(IJ.d2s(currRM.model.getParamNumber(),0) +SEP_CHAR);
		double[] params = currRM.model.getParams();
    	for (int i=0; i<params.length; i++) {
    		sb.append(IJ.d2s(params[i],3) + SEP_CHAR);
    	}
    	sb.append(currRM.psf.title + SEP_CHAR);
    	sb.append(currRM.psf.desc + SEP_CHAR);
    	sb.append(IJ.d2s(currRM.psf.focusParam,3) + SEP_CHAR);
		sb.append(ddwin.taInfo.getText() + SEP_CHAR);
		
		// Fitting Results
		sb.append(DDCurveFitter.fitList[0] + SEP_CHAR);
    	sb.append(IJ.d2s(ddp.getError(),2) + SEP_CHAR);
		sb.append(IJ.d2s(fitter.getFitGoodness(),6) + SEP_CHAR);
		sb.append(IJ.d2s(ddp.getCorrel(),4) + SEP_CHAR);
		sb.append(IJ.d2s(ddp.getPerArea(),2) + SEP_CHAR);
		int nParamsFree = fitter.getNumParams();
		double dataParamRatio = (nParamsFree==0)?Double.NaN:((double)currLS.lsInts.length/nParamsFree);
		sb.append(IJ.d2s(dataParamRatio,2) + SEP_CHAR);
		sb.append(IJ.d2s(nParamsFree,0) + SEP_CHAR);
		int nParamsTotal = currRM.model.getParamNumber() + 2*currRM.repts.length;
		sb.append(IJ.d2s(nParamsTotal,0) + SEP_CHAR);
		sb.append(("" + fitter.maxError) + SEP_CHAR);
		sb.append(IJ.d2s(fitter.numIter,0) + SEP_CHAR);
		sb.append(IJ.d2s(fitter.maxIter,0) + SEP_CHAR);
		sb.append(IJ.d2s(fitter.setRestarts,0) + SEP_CHAR);
		sb.append(encodePStates() + SEP_CHAR);
		sb.append(IJ.d2s(fitter.max_int_search,3) + SEP_CHAR);
		sb.append(IJ.d2s(fitter.max_pos_search,3) + SEP_CHAR);
		sb.append(IJ.d2s(RepeatModel.MAX_OVERHANG,0) + SEP_CHAR);
		sb.append(fitter.info + SEP_CHAR);
    	return new String(sb);
	}
	
	public String getLSIntsLabel() {
		StringBuffer sb = new StringBuffer();
		sb.append("X" + SEP_CHAR);
		sb.append("I_Obs" + SEP_CHAR);
		return new String(sb);
	}
	
	public String getLSInts(int n) {
		StringBuffer sb = new StringBuffer();
		sb.append(IJ.d2s(n,0) + SEP_CHAR);
		sb.append(IJ.d2s(currLS.lsInts[n],4) + SEP_CHAR);
		return new String(sb);
	}
	
	public String getDDIntsLabel() {
		StringBuffer sb = new StringBuffer(getLSIntsLabel());
		sb.append("I_Calc" + SEP_CHAR);
		sb.append("I_Res" + SEP_CHAR);
		return new String(sb);
	}
	
	public String getDDInts(int n) {
		StringBuffer sb = new StringBuffer(getLSInts(n));
		sb.append(IJ.d2s(currRM.calcInts[n],4) + SEP_CHAR);
		sb.append(IJ.d2s(manager.ddp.repeatResidualPixels[n].y,4) + SEP_CHAR);
    	return new String(sb);
	}
	
	public String getSarcViewLabel() {
		StringBuffer sb = new StringBuffer(getDDIntsLabel());
		sb.append("Distance From Nearest Z" + SEP_CHAR);
		sb.append("Distance From Nearest M" + SEP_CHAR);
		sb.append("Sarcomere Length" + SEP_CHAR);
		sb.append("Sarcomere #" + SEP_CHAR);
		sb.append("Sarcomere Pixel" + SEP_CHAR);
		return new String(sb);
	}
	
	public String getSarcViewInts(int n) {
		StringBuffer sb = new StringBuffer(getDDInts(n));
		sb.append(IJ.d2s(getDistNearZ(n),2) + SEP_CHAR);
		sb.append(IJ.d2s(getDistNearM(n),2) + SEP_CHAR);
		sb.append(IJ.d2s(getSarcLength(n),2) + SEP_CHAR);
		sb.append(IJ.d2s(getSarcNum(n),0) + SEP_CHAR);
		sb.append(IJ.d2s(getSarcPix(n),0) + SEP_CHAR);
		return new String(sb);
	}
	
	public double getDistNearZ(int n) {
		return ddp.modelResidualPixels[n].x;
	}
	
	public double getDistNearM(int n) {
		int sarc = getSarcNum(n);
		if (sarc<0 || sarc>=currRM.repts.length-1) return Double.NaN;
		double m = (currRM.repts[sarc+1].x + currRM.repts[sarc].x)/2;
		return n-m;
	}
	
	public double getSarcLength(int n) {
		int sarc = getSarcNum(n);
		if (sarc<0 || sarc>=currRM.repts.length-1) return Double.NaN;
		return currRM.repts[sarc+1].x-currRM.repts[sarc].x;
	}
	
	public int getSarcNum(int n) {
		int sarc = -1;
		for (int i=0; i<currRM.repts.length; i++) {
			if (currRM.repts[i].x<=n) sarc++;
		}
		return sarc;
	}
	
	public int getSarcPix(int n) {
		int sarc = getSarcNum(n);
		if (sarc<0 || sarc>=currRM.repts.length-1) return -1;
		return (int)Math.floor(n-currRM.repts[sarc].x);
	}
	
/*	public void exportData() {
		boolean[] saves = confirmExport();
		if(saves[0]) {
			// Save Cataloged Image
			IJ.showStatus("Saving Cataloged Image...");
			catImageInfo.save();
		}
		if(saves[1]) {
			// Save SubImage
			IJ.showStatus("Saving SubImage...");
			saveSubImage(currLS.imp.getOriginalFileInfo().directory);
		}
		if (saves[2]) {
			// Save Intensities in a text file
			IJ.showStatus("Saving Intensities...");
			saveIntensities(intensitySavePath);
			doSarcView();
		}
		if (saves[3]) {
			// Save Results
			IJ.showStatus("Saving Results...");
			exportResults();
//			results.getTextPanel().saveAs(resultsSavePath);
		}
	}
*/
/*	public boolean[] confirmExport() {		
		ImagePlus imp = manager.getImage();
		if (catImageInfo==null && imp!=null) {
			// Check to see if this image was already cataloged by looking for text file in the same directory.
			String dir = imp.getOriginalFileInfo().directory;
			String fname = imp.getShortTitle();
			String path = dir + fname + CatalogedImageInfo.POSTFIX;
			File f = new File(path);
			catImageInfo = new CatalogedImageInfo(dir,fname,f.exists());
		}
		String catImageStr = catImageInfo.confirm();
		catalogPath = this.catImageInfo.getSavePath();
		
		
    	FileInfo fi = currLS.subImage.getFileInfo();
    	fi.directory = currLS.imp.getOriginalFileInfo().directory;
    	fi.fileName = currLS.subImage.getTitle();
    	fi.fileFormat = FileInfo.TIFF;
    	fi.description = LSDDecon_Manager.DESCRIPTION;
    	fi.info = "skewAngle=" + currLS.skewAngle
    			+ "; regionWidths=[" + currLS.regions[0].rect.height
    			+ "," + currLS.regions[1].rect.height
    			+ "," + currLS.regions[2].rect.height
    			+ "]"
    			+ "; line: x1d,y1d=(" + IJ.d2s(currLS.line.x1d,2)
    			+ "," + IJ.d2s(currLS.line.y1d,2)
    			+ "); x2d,y2d=(" + IJ.d2s(currLS.line.x2d,2)
    			+ "," + IJ.d2s(currLS.line.y2d,2)
    			+ ")";
    	currLS.subImage.setFileInfo(fi);
		FileInfo ofi = currLS.subImage.getOriginalFileInfo();
		String subImageInfoStr = "Name= " + ofi.fileName
				+ "\nWidth = " + ofi.width + " pixels (" + IJ.d2s(ofi.width*ofi.pixelWidth,2) + " microns)"
				+ "\nHeight = " + ofi.height + " pixels (" + IJ.d2s(ofi.height*ofi.pixelHeight,2) + " microns)"
				+ "\nInfo = " + ofi.info
				+ "\nDescription = " + ofi.description
				;
		this.subImageSavePath = ofi.directory + ofi.fileName;


		String intensityStr;
		if (ddwin==null) {
			// No DDecon, so only linescan intensities will be written
			intensityStr = "Only Linescan data avaliable.";
		}
		else {
			// Observed Linescan Intensities
			double[] Iobs = currLS.lsInts;
			int[] intCalc = {currLS.regions[0].mode, currLS.regions[1].mode, currLS.regions[2].mode};
			int corrCalc = currLS.bgCorrIndex;
			
			// Modeled DDecon Intensities
			int nr = currRM.repts.length;
			Point2D.Double[] repts = currRM.repts;
			intensityStr = "Number of points=" + Iobs.length
					+ "\nBG1calc=" + intCalc[0]
					+ "; LScalc=" + intCalc[1]
					+ "; BG2calc=" + intCalc[2]
					+ "\nBGcorrection=" + corrCalc
					+ "\nNumber of Repeats=" + nr
					+ "\nAverage Sarcomere Length=" + IJ.d2s((repts[nr-1].x-repts[0].x)/(nr-1),2)+" pixels.";
					;
			intensitySavePath = currLS.imp.getOriginalFileInfo().directory +
								currLS.subImage.getShortTitle() +
								"_int.txt";
		}

		
		String resultsStr = "";
		resultsSavePath = currLS.imp.getOriginalFileInfo().directory + "Distributed Deconvolution Results.txt";

		
		// Open confirmation dialog
		GenericDialog gd = new GenericDialog("Confirm Data Export");
		gd.addMessage("Original Image");
		gd.addMessage(catImageStr);
		gd.addCheckbox("Catalog Image",false);
		gd.addStringField("Location: ",catalogPath,90);
		
		gd.addMessage("LineScan SubImage");
		gd.addMessage(subImageInfoStr);
		gd.addCheckbox("Save",true);
		gd.addStringField("Location: ",subImageSavePath,90);
		
		gd.addMessage("LineScan, Modeled, and SarcView Intensities");
		gd.addMessage(intensityStr);
		gd.addCheckbox("Save",false);
		gd.addStringField("Location: ",intensitySavePath,90);
		
		gd.addMessage("Results");
		gd.addMessage(resultsStr);
		gd.addCheckbox("Save",false);
		gd.addStringField("Location: ",resultsSavePath,90);
		gd.showDialog();

		boolean[] saves = {false,false,false,false};
		if (gd.wasCanceled()) return saves;
		saves[0] = gd.getNextBoolean();
		catalogPath = gd.getNextString();
		saves[1] = gd.getNextBoolean();
		subImageSavePath = gd.getNextString();
		saves[2] = gd.getNextBoolean();
		intensitySavePath = gd.getNextString();
		saves[3] = gd.getNextBoolean();
		resultsSavePath = gd.getNextString();
		Prefs.set("LSDDecon.lastResultsPath",resultsSavePath);
		return saves;
	}
*/	
/*	public String getCatalogInfo() {
		ImagePlus imp = manager.getImage();
		if (catImageInfo==null && imp!=null) catalogImage(imp);
		String s = "" + this.catImageInfo.confirm();
		catalogPath = this.catImageInfo.getSavePath();
		return s;
	}
*/	
	public void catalogImage(ImagePlus imp) {
//		ImagePlus imp = getImage();
		catImageInfo = new CatalogedImageInfo(imp);
		if (catImageInfo.isSaved()) catImageInfo.load();
		else catImageInfo.edit();
		// Check to see if this image was already cataloged by looking for text file in the same directory.
//		String dir = imp.getOriginalFileInfo().directory;
//		String fname = imp.getShortTitle();
//		String path = dir + fname + CatalogedImageInfo.POSTFIX;
//		File f = new File(path);
//		catImageInfo = new CatalogedImageInfo(dir,fname,f.exists());
	}
	
/*	public void saveIntensities(String path) {
    	String name = "Intensities";
    	String intHeadings = "Position\tI_Obs\tI_Calc\tCalcData";
    	String intValues;
    	TextWindow win = new TextWindow(name,"",300,200);
    	TextPanel tp = win.getTextPanel();
   		tp.setColumnHeadings(" " + intHeadings);

    	for (int i=0; i<currLS.lsInts.length; i++) {
       		intValues = i + "\t" + IJ.d2s(currLS.lsInts[i],4) + "\t" + IJ.d2s(currRM.calcInts[i],4) + "\t";
    		if (i==0) intValues = intValues + currLS.regions[0].mode + "\t";
    		if (i==1) intValues = intValues + currLS.regions[1].mode + "\t";
    		if (i==2) intValues = intValues + currLS.regions[2].mode + "\t";
    		if (i==3) intValues = intValues + currLS.bgCorrIndex + "\t";
    		tp.appendLine(intValues);
    	}
    	tp.saveAs(path);
    	win.close();
	}
*/	
/*	public boolean doSarcView() {
		IJ.log("Starting sarcView");
//		ddi.pStates = getParamStates(); // set the current param fix states in ddecon processor
//		if (ddi==null) ddi = new DDinfo();
		SarcData[] sd = prepSarcView();
		
		// Make new ResultWindow with the view results
		double p;
		double q;
		TextWindow tw = new TextWindow("","",100,100);
		WindowManager.addWindow(tw);
		tw.setVisible(false);
		TextPanel tp = tw.getTextPanel();
		String path;
		String headings = "SarcData\tDistance From Z\tDistance From M\tIobs\tIcalc\tIres";
		tp.setColumnHeadings(headings);
		String lineOut;
		for (int n=0; n<sd.length; n++) {
			for (int i=0; i<sd[n].npix; i++) {
				lineOut = " " + "\t";
				p = sd[n].length/2 - sd[n].xVals[i];
				q = sd[n].iVals[0][i]-sd[n].iVals[1][i];
				if (i==0) lineOut = IJ.d2s(n,0) + "\t";
				if (i==1) lineOut = IJ.d2s(sd[n].length,4) + "\t";
				if (i==2) lineOut = IJ.d2s(currRM.repts[n].y,4) + "\t";
				if (i==3) lineOut = IJ.d2s(currRM.repts[n+1].y,4) + "\t";
				lineOut = lineOut + IJ.d2s(sd[n].xVals[i],4) + "\t"
								+ IJ.d2s(p,4) + "\t" 
								+ IJ.d2s(sd[n].iVals[0][i],4) + "\t" 
								+ IJ.d2s(sd[n].iVals[1][i],4) + "\t"
								+ IJ.d2s(q,4) + "\t";
				tp.appendLine(lineOut);
			}
			String title = "Sarcomere-"+n;
			tw.setTitle(title);
			tw.show();
//			results.show(title);
			path = currLS.imp.getOriginalFileInfo().directory + currLS.subImage.getShortTitle() + "_sarc"+n+".txt";
			IJ.log("DDwindow.doSarcView: path= "+path);
//			path="";
			PrintWriter pw = null;
			try {
				FileOutputStream fos = new FileOutputStream(path);
				BufferedOutputStream bos = new BufferedOutputStream(fos);
				pw = new PrintWriter(bos);
			}
			catch (IOException e) {
				IJ.write("" + e);
			}
			tp.save(pw);
			pw.close();
			tp.selectAll();
			tp.clearSelection();
			tp.resetSelection();
		}
		tw.close();

		
		Prefs.set("LSDDecon.gridIntvl",DDplotter.GRID_LINE_INTERVAL); // XXX not user adjustable yet
		Prefs.set("LSDDecon.autoNorm",true); // XXX Not user adjustable yet
		Prefs.set("LSDDecon.pwpp",DDplotter.PLOT_W_PERPIXEL); // XXX Not user adjustable yet
		Prefs.set("LSDDecon.minTFL",fitter.minTFL);
		Prefs.set("LSDDecon.maxTFL",fitter.maxTFL);

		Prefs.set("LSDDecon.maxOverhang",repeatModel.MAX_OVERHANG); // XXX not user adjustable yet
		Prefs.set("LSDDecon.defModIndex",currRM.cModIndex); // XXX Set to current model used
		Prefs.set("LSDDecon.pSearchR",fitter.max_pos_search);
		Prefs.set("LSDDecon.wSearchR",fitter.max_int_search);
		double aveSL = repeatModel.REPEAT_SPACING;
		int nr = currRM.repts.length;
		if (nr > 1)
			aveSL = (currRM.repts[nr-1].x - currRM.repts[0].x)/(nr-1); // average sarcomere length
		Prefs.set("LSDDecon.aveSL", aveSL);
//		Prefs.set("LSDDecon.linescanID",currLS.linescanID);
//		Prefs.set("LSDDecon.date",currLS.date);
//		Prefs.set("LSDDecon.sample",currLS.sample);
//		Prefs.set("LSDDecon.fluor",currLS.fluor);
//		Prefs.set("LSDDecon.exposure",currLS.expTime);
		return true;
	}
*/	
/*	public SarcData[] prepSarcView() {
		double[] oy = currLS.lsInts;
//		double[] oy = this.pf;
		double[] my = currRM.calcInts;
		double[] ix = ddp.getXs(ddp.repeatResidualPixels);
		double[] xZD = ddp.getXs(currRM.repts); // X-coordinates of Z lines
		
		// Number of Sarcomeres
		int nSarc = currRM.repts.length-1;
		
		// Populate sarcomeres
		SarcData[] sd = new SarcData[nSarc];
		for (int i=0; i<nSarc; i++) {
			Point2D.Double[] szd = {currRM.repts[i],currRM.repts[i+1]};
			int iZL = (int)(float)Math.ceil(xZD[i]);
			int npix = (int)(float)Math.floor(xZD[i+1]) - iZL + 1;
			double[] xVals = new double[npix];
			double[] iObs = new double[npix];
			double[] iCalc = new double[npix];
			for (int n=0; n<npix; n++) {
				xVals[n] = ix[n+iZL]-xZD[i];
				iObs[n] = oy[n+iZL];
				iCalc[n] = my[n+iZL];
			}
			sd[i] = new SarcData(szd,xVals,iObs,iCalc);
		}
		return sd;
	}
*/	
/*	public void exportResults() {
//		ddi.pStates = getParamStates(); // get the current param fix states in ddecon processor
//		ddi.lsi.linescanID = lfPFID.getText();
//		ddi.date = lfDate.getText();
//		ddi.sample = lfSample.getText();
//		ddi.fluor = lfFluor.getText();
//		try {
//			ddi.expTime = Double.parseDouble(lfExpT.getText());
//				} catch (Exception NumberFormatException) {ddi.expTime = 0;}
//		ddi.model = chModel.getSelectedItem();
//		ddi.infoDDecon = taInfo.getText();
		
		
		// Make sure results window is open and set
		if (results == null) {
			String resultsName = "Distributed Deconvolution Results";
			Frame win = WindowManager.getFrame(resultsName);
			if (win!=null){
				// set an open results window from a previous analysis.
				results = (TextWindow)win;
			}
			else {
				String defPath = "";
				resultsSavePath = Prefs.get("LSDDecon.resultsSavePath",defPath);
				// open up lastUsedResults (XXX currently set to null, call Prefs.set somewhere)
				results = manager.loader.openResults(resultsSavePath);
				
				// Do nothing if canceled or file does not open
				if (manager.results==null) {
		    		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		    		int h = 200;
		    		manager.results = new TextWindow(resultsName,"",d.width,h);
		    		manager.results.setLocation(0,d.height-h-30);
		    		manager.results.addWindowListener(manager);
		    	}
			}
		}
		summarize();
		saveResults(resultsSavePath);
		
		Prefs.set("LSDDecon.gridIntvl",DDplotter.GRID_LINE_INTERVAL); // XXX not user adjustable yet
		Prefs.set("LSDDecon.autoNorm",true); // XXX Not user adjustable yet
		Prefs.set("LSDDecon.pwpp",DDplotter.PLOT_W_PERPIXEL); // XXX Not user adjustable yet
		Prefs.set("LSDDecon.minTFL",fitter.minTFL);
		Prefs.set("LSDDecon.maxTFL",fitter.maxTFL);

		Prefs.set("LSDDecon.maxOverhang",repeatModel.MAX_OVERHANG); // XXX not user adjustable yet
		Prefs.set("LSDDecon.defModIndex",currRM.cModIndex); // XXX Set to current model used
		Prefs.set("LSDDecon.pSearchR",fitter.max_pos_search);
		Prefs.set("LSDDecon.wSearchR",fitter.max_int_search);
		double aveSL = repeatModel.REPEAT_SPACING;
		int nr = currRM.repts.length;
		if (nr > 1)
			aveSL = (currRM.repts[nr-1].x - currRM.repts[0].x)/(nr-1); // average sarcomere length
		Prefs.set("LSDDecon.aveSL", aveSL);
//		Prefs.set("LSDDecon.linescanID",currLS.linescanID);
//		Prefs.set("LSDDecon.date",currLS.date);
//		Prefs.set("LSDDecon.sample",currLS.sample);
//		Prefs.set("LSDDecon.fluor",currLS.fluor);
//		Prefs.set("LSDDecon.exposure",currLS.expTime);
	}
*/	
    /** Returns headings for Distributed Deconvolution Results Window
     * Called by:
     * 	DDinfo.summarize(Boolean)
     * 
     * @return tab-delimited string of heading names
     */
/*    public String getLabelHeadingString() {
    	int maxZ = repeatModel.MAX_NUMBER_REPEATS;
    	String[] labels1 = {"Img FileName","Linescan ID","Date","Sample","Fluor","Exp. Time",
    			"Model","Error","GoodFit","Corr. Coeff.","Per. Area","Data Points","Z Lines"};
    	String[] Zlabels = new String [2*maxZ];
    	for (int i=0; i<maxZ; i++) {
    		Zlabels[i] = "Z" + (i+1) + " Pos\t";
    		Zlabels[i+maxZ] = "Z" + (i+1) + " Wght\t";
    	}
    	String[] labels2x;
    	if (Double.parseDouble(LSDDecon_Manager.VERSION)>=3) {
    		String[] labels2 = {"Fit Method","Data/FreeParams","Free Params","Total Params",
    			"Tolerance","Num. Iter.","Max. Iter","Restarts","Params Fixed",
    			"Int. Search Rad.","Pos. Search Rad.","Max. Overhang",
    			"LS Length","LS Width(Total)","LS angle",
    			"BG1 Width","rawLS Width","BG2 Width",
    			"BG1 Calc","rawLS Calc","BG2 Calc","LS correction",
    			"ROI X1","ROI X2","ROI Y1","ROI Y2",
    			"Img FileName","Img Directory","Img Format","Img Type",
    			"Img Frame","Img Stack Size","Img Height","Img Width",
    			"Img Pixel Depth","Img Pixel Height","Img Pixel Width",
    			"Description","SkewAngle","Fit Info","DDecon Info","LS Info","Img Info"};
    		labels2x=labels2;
    	}
    	else {
    		String[] labels2 = {"Fit Method","Data/FreeParams","Free Params","Total Params",
    			"Tolerance","Num. Iter.","Max. Iter","Restarts","Params Fixed",
    			"Int. Search Rad.","Pos. Search Rad.","Max. Overhang",
    			"LS Length","LS Width(Total)","LS angle",
    			"BG1 Width","rawLS Width","BG2 Width",
    			"BG1 Calc","rawLS Calc","BG2 Calc","LS correction",
    			"ROI X1","ROI X2","ROI Y1","ROI Y2",
    			"Img FileName","Img Directory","Img Format","Img Type",
    			"Img Frame","Img Stack Size","Img Height","Img Width",
    			"Img Pixel Depth","Img Pixel Height","Img Pixel Width",
    			"Description","Fit Info","DDecon Info","LS Info","Img Info"};
    		labels2x=labels2;
    	}
    	StringBuffer output = new StringBuffer(200);
    	output.append("\t");
    	for (int i=0; i<labels1.length; i++) {
    		output.append(labels1[i] + "\t");
    	}
    	for (int j=0; j<currRM.model.getParamNames().length; j++) {
    		output.append(currRM.model.getParamNames()[j] + "\t");
    	}
    	for (int j=0; j<2*maxZ; j++) {
    		output.append(Zlabels[j]);
    	}
    	for (int i=0; i<labels2x.length; i++) {
    		output.append(labels2x[i] + "\t");
    	}
    	return new String(output);
    }
*/    
    /** Summarizes DDinfo for Distributed Deconvolution Results Window
     * Called by:
     * 		DDWindow.actionPerformed(ActionEvent)
     * 
     * @param appendVals: flag to output intensity values, NOT IMPLEMENTED
     */
/*    public void summarize() {
    	String ddeconHeadings = getLabelHeadingString();
    	String ddeconValues = getLabelString();
    	if (results != null) {
    		TextPanel tp = results.getTextPanel();
    		if (tp.getColumnHeadings().equals("")) {
    			tp.setColumnHeadings(" " + ddeconHeadings);;
    		}
    		int index = 1 + tp.getLineCount();
    		tp.appendLine(index + ddeconValues);
    	}
    }
*/    
    /** Returns values of DDinfo object for Distributed Deconvolution Results Window
     * Called by:
     * 	DDinfo.summarize(Boolean) 
     * 
     * @return tab-delimited string of DDinfo values
     */
/*    public String getLabelString() {
		int nParamsTotal = currRM.model.getParamNumber() + 2*currRM.repts.length;
		int nParamsFree = 0;
		boolean[] modelParamStates = currRM.model.getParamStates();
		boolean[][] reptParamStates = currRM.reptFixStates;
		if (modelParamStates!=null) {
			int index = 0;
			for (int i=0; i<modelParamStates.length; i++) {
				if (!modelParamStates[i]) index++;
			}
			for (int i=0; i<reptParamStates[0].length; i++) {
				if (!reptParamStates[0][i]) index++;
				if (!reptParamStates[1][i]) index++;
			}
			nParamsFree = index;
		}
//		int nParamsFree = ddi.getNumFreeParams();
		double dataParamRatio = (nParamsFree==0)?Double.NaN:((double)currLS.lsInts.length/nParamsFree);
    	int maxRepts = repeatModel.MAX_NUMBER_REPEATS;
    	String infoFit = IJ.d2s(fitter.getSumResidualsSqr(),2)+ " " + IJ.d2s(fitter.getChiSquared(),2)+ " " + fitter.info;
    	FileInfo ofi = currLS.imp.getOriginalFileInfo();
    	
    	StringBuffer sb = new StringBuffer(200);
    	sb.append("\t");
    	sb.append(ofi.fileName + "\t");
    	sb.append(ddwin.lfPFID.getText() + "\t");
    	sb.append(ddwin.lfDate.getText() + "\t");
    	sb.append(ddwin.lfSample.getText() + "\t");
    	sb.append(ddwin.lfFluor.getText() + "\t");
    	sb.append(ddwin.lfExpT.getText() + "\t");
    	sb.append(currRM.model.getTitle() + "\t");

    	sb.append(IJ.d2s(ddp.getError(),2) + "\t");
    	sb.append(IJ.d2s(fitter.getFitGoodness(),6) + "\t");
    	sb.append(IJ.d2s(ddp.getCorrel(),4) + "\t");
    	sb.append(IJ.d2s(ddp.getPerArea(),2) + "\t");
    	sb.append(currLS.lsInts.length + "\t");
    	sb.append(currRM.repts.length + "\t");

    	double[] modelParams = currRM.model.getParams();
    	for (int i=0; i<modelParams.length; i++) {
    		sb.append(IJ.d2s(modelParams[i],2) + "\t");
    	}
    	for (int i=0; i<maxRepts; i++) {
    		sb.append((i<currRM.repts.length?IJ.d2s(currRM.repts[i].x,2):"-") + "\t");
    	}
    	for (int i=0; i<maxRepts; i++) {
    		sb.append((i<currRM.repts.length?IJ.d2s(currRM.repts[i].y,2):"-") + "\t");
    	}

    	sb.append(DDCurveFitter.fitList[0] + "\t");
    	sb.append(IJ.d2s(dataParamRatio,2) + "\t");
    	sb.append(nParamsFree + "\t");
    	sb.append(nParamsTotal + "\t");
    	sb.append(fitter.maxError + "\t"); //errTol
    	sb.append(fitter.numIter + "\t");
    	sb.append(fitter.maxIter + "\t");
    	sb.append(fitter.setRestarts + "\t"); // restarts
    	sb.append(encodePStates() + "\t");
    	sb.append(fitter.max_int_search + "\t");
    	sb.append(fitter.max_pos_search + "\t");
    	sb.append(repeatModel.MAX_OVERHANG + "\t");

    	sb.append(currLS.lsLength + "\t");
    	sb.append(currLS.lsWidth + "\t");
    	sb.append(IJ.d2s(180*currLS.lsAngle/Math.PI,2) + "\t");
    	sb.append(currLS.regions[0].rect.height + "\t");
    	sb.append(currLS.regions[1].rect.height + "\t");
    	sb.append(currLS.regions[2].rect.height + "\t");
    	sb.append(currLS.regions[0].mode + "\t");
    	sb.append(currLS.regions[1].mode + "\t");
    	sb.append(currLS.regions[2].mode + "\t");
      	sb.append(LSprocessor.BG_MODES[currLS.bgCorrIndex] + "\t");

      	sb.append(IJ.d2s(currLS.line.x1d,2) + "\t"); sb.append(IJ.d2s(currLS.line.y1d,2) + "\t");
    	sb.append(IJ.d2s(currLS.line.x2d,2) + "\t"); sb.append(IJ.d2s(currLS.line.y2d,2) + "\t");

    	sb.append(ofi.fileName + "\t");
    	sb.append(ofi.directory + "\t");
        int fileFormat = ofi.fileFormat;
        String fileFormatStr = "";
    	switch (fileFormat) {
			case FileInfo.UNKNOWN: fileFormatStr = "unknown"; break;
			case FileInfo.RAW: fileFormatStr = "raw"; break;
			case FileInfo.TIFF: fileFormatStr = "tif"; break;
			case FileInfo.GIF_OR_JPG: fileFormatStr = "gif or jpg"; break;
			case FileInfo.FITS: fileFormatStr = "fits"; break;
			case FileInfo.BMP: fileFormatStr = "bmp"; break;
			case FileInfo.DICOM: fileFormatStr = "dicom"; break;
			case FileInfo.ZIP_ARCHIVE: fileFormatStr = "zip"; break;
			case FileInfo.PGM: fileFormatStr = "pgm"; break;
			default: fileFormatStr = ""; break;
    	}
    	sb.append(fileFormatStr + "\t");
    	int fileType = ofi.fileType;
    	String fileTypeStr = "";
    	switch (fileType) {
			case FileInfo.GRAY8: fileTypeStr = "byte"; break;
			case FileInfo.GRAY16_SIGNED: fileTypeStr = "short"; break;
			case FileInfo.GRAY16_UNSIGNED: fileTypeStr = "ushort"; break;
			case FileInfo.GRAY32_INT: fileTypeStr = "int"; break;
			case FileInfo.GRAY32_UNSIGNED: fileTypeStr = "uint"; break;
			case FileInfo.GRAY32_FLOAT: fileTypeStr = "float"; break;
			case FileInfo.COLOR8: fileTypeStr = "byte+lut"; break;
			case FileInfo.RGB: fileTypeStr = "RGB"; break;
			case FileInfo.RGB_PLANAR: fileTypeStr = "RGB(p)"; break;
			case FileInfo.RGB48: fileTypeStr = "RGB48"; break;
			case FileInfo.BITMAP: fileTypeStr = "bitmap"; break;
			case FileInfo.ARGB: fileTypeStr = "ARGB"; break;
			case FileInfo.BGR: fileTypeStr = "BGR"; break;
			case FileInfo.BARG: fileTypeStr = "BARG"; break;
			default: fileTypeStr = ""; break;
    	}
    	sb.append(fileTypeStr + "\t");
    	sb.append(currLS.slice + "\t");
    	sb.append(ofi.nImages + "\t");
    	sb.append(ofi.height + "\t");
    	sb.append(ofi.width + "\t");
    	sb.append(ofi.pixelDepth + "\t");
    	sb.append(ofi.pixelHeight + "\t");
    	sb.append(ofi.pixelWidth + "\t");
    	sb.append(ofi.description + "\t"); // This description contains the DDecon version
    	
    	// Append additional parameters after here
    	sb.append(IJ.d2s(currLS.skewAngle,2) + "\t");
    	
    	// Append additional parameters before here
    	sb.append(infoFit + "\t");
    	sb.append(ddwin.taInfo.getText() + "\t");
    	
//    	System.out.println("DDinfo.getLabelString: lsi.imgInfo= "+lsi.imgInfo);
//    	System.out.println("DDinfo.getLabelString: lsi.fi.info= "+lsi.fi.info);

    	int extendedHeaderIndex = ofi.info.indexOf("ExtendedHeader");
    	String shortInfo = ofi.info.substring(0,extendedHeaderIndex);
    	String imgInfo = shortInfo.replace('\n',';');
    	sb.append(lswin.taInfo.getText() + "\t");
    	sb.append(imgInfo + "\t");
//    	sb.append(lsi.infoLS + "\t"); sb.append(lsi.imp.getFileInfo().info + "\t");
//    	sb.append(lsi.infoLS + "\t"); sb.append(lsi.infoImage + "\t");
    	return new String(sb);
    }
*/    
    /** Encodes parameter states (FIXED or FREE) as a string for writing
     * Called by:
     * 	DDinfo.getLabelString() 
     * 
     * @return string of encoded parameter states
     */
    public String encodePStates() {
    	boolean[] modelParamStates = currRM.model.getParamStates();
    	int nmp = modelParamStates.length;
    	char[] f = new char [nmp];
   		for (int j=0; j<nmp; j++) {
   			f[j] = (modelParamStates[j]) ? FIXED : FREE;
   		}
    	String modelState = new String(f);
    	
   		boolean[][] pStates = currRM.reptFixStates;
    	int nr = currRM.repts.length;
   		char[] r0 = new char[nr];
   		for (int j=0; j<nr; j++) {
   			r0[j] = (pStates[0][j]) ? FIXED : FREE;
   		}
    	String posState = new String(r0);
    	
   		char[] r1 = new char[nr];
   		for (int j=0; j<nr; j++) {
   			r1[j] = (pStates[1][j]) ? FIXED : FREE;
   		}
    	String wghtState = new String(r1);

    	return "<" + modelState + "/" + posState + "/" + wghtState + ">";
    }

/*	public void saveResults(String path) {
		if (path.equals("")) {
			SaveDialog sd = new SaveDialog("Save as Text", results.getTitle(), ".txt");
			String file = sd.getFileName();
			if (file == null) return;
			path = sd.getDirectory() + file;
		}
		PrintWriter pw = null;
		try {
			FileOutputStream fos = new FileOutputStream(path);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			pw = new PrintWriter(bos);
		}
		catch (IOException e) {
			//IJ.write("" + e);
			return;
		}
		results.getTextPanel().save(pw);
		pw.close();
		IJ.showStatus("");
		Prefs.set("LSDDecon.resultsSavePath",path);
	}
*/	
}
