/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//import ij.IJ;
import ij.IJ;
import ij.ImagePlus;
import ij.Prefs;
//import ij.Prefs;
//import ij.WindowManager;
import ij.gui.*;
//import ij.io.*;
//import ij.measure.Calibration;
import ij.process.ImageProcessor;
import ij.util.Tools;

import java.awt.Color;
import java.awt.Component;
import java.awt.Scrollbar;
//import java.awt.Font;
//import java.awt.FontMetrics;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
//import java.awt.event.KeyEvent;
//import java.awt.event.MouseEvent;
//import java.awt.event.MouseMotionListener;
//import java.awt.geom.Line2D;

public class LSprocessor {
//public class LSprocessor implements MouseMotionListener {

	public static final float[] ZERO = {0};
	public static final int BG_MIN = 0;
	public static final int BG_AVE = 1;
	public static final int BG_1 = 2;
	public static final int BG_2 = 3;
	public static final int BG_CONST = 4;
	public static final String [] BG_MODES ={"LSraw - min(BG1,BG2)"
											,"LSraw - ave(BG1,BG2)"
											,"LSraw - BG1"
											,"LSraw - BG2"
											,"LSraw - [constant]"};
	public static int numReg = 3;
	public ImagePlus imp;
	public String lsName;
	public Line line;
	public Point2D.Double[] linePts = new Point2D.Double[2];
	public int slice;
	public int lsLength;
	public LSRegion[] regions = new LSRegion[numReg];
	public int lsWidth;
	public double lsAngle;
	public double skewAngle;
	public ImagePlus subImage;
	public ImagePlus rawPlotImp;
	public ImagePlus corrPlotImp;
	public int bgCorrIndex;
//	public int bgConst = 0;
	public double[] lsInts;
	public String lsInfo;
	
//	LSinfo lsi;
	LSwindow lswin;
//	ImagePlus imp;
	ImagePlus plotImp, plotImp2;
	ImageProcessor ipSub;
//	ImageProcessor ipMain;
	ImageCanvas canvas;
//	public LineScanCanvas[] cans = new LineScanCanvas[3];
	LSDDecon_Manager manager;

	Rectangle[] regionRects = new Rectangle [3];
//	Rectangle bg1Rect, lsRect, bg2Rect;
//	String bgMode;
//	boolean isBgConst = false;
//	double bgConstValue = 0;
//	int bgModeIndex = 0;
//	static double PLOT_W_PERPIXEL = 4;
	double subScale = LSwindow.PLOT_W_PERPIXEL;
//	private double [] corrLS;
	public double [] xvals;

//	String linescanID;
//	int currSlice;
//	Font f;
//	FontMetrics fm;
	boolean isFigure = true;
	
	public LSprocessor (ImagePlus imp, Line line) {
		this(imp,line,imp.getCurrentSlice(),0,new int[numReg],new int[numReg],-1,"");
	}
	
	public LSprocessor (ImagePlus imp, LineRegion lr) {
		this(imp,lr.line,imp.getCurrentSlice(),lr.skewAngle,lr.regWidths,lr.regModes,lr.bgCorrIndex,"");
	}
	
	public LSprocessor (ImagePlus imp, Line line, int slice, double skewAngle, int[] regWidths, int[] regModes, int bgCorrIndex, String info) {
		this.imp = imp;
		if (line.getName()==null) setLineName(line);
		String lsID = getLSID(line.getName());
		lsName = imp.getShortTitle() + "_" + lsID;
//		lsName = imp.getShortTitle() + "_" + slice + "_" + lsID;
//		lsName = imp.getShortTitle() + "_" + slice + "_" + line.getName();
		linePts[0] = new Point2D.Double(line.x1d,line.y1d);
		linePts[1] = new Point2D.Double(line.x2d,line.y2d);
//		this.line = line;
		lsLength = (int)Math.round(line.getRawLength());
//		System.out.println("LSProcessor.<init>: lsLength="+lsLength);

		int tempW = 0;
		for (int i=0; i<regions.length; i++) {
			if (regWidths[i]==0) regWidths[i] = (int)Prefs.get("LSDDecon.rw"+i,10);
//			System.out.println("LSProcessor.<init>: rw["+i+"]="+regWidths[i]);
			if (regModes[i]==0) regModes[i] = (int)Prefs.get("LSDDecon.rm"+i,LSRegion.AVE);
			Rectangle r = new Rectangle(0,tempW,lsLength,regWidths[i]);
			regions[i] = new LSRegion(r,regModes[i]);
			tempW += regWidths[i];
		}
		lsWidth = tempW;
		
		lsAngle = Math.atan2(line.y2d-line.y1d,line.x2d-line.x1d);
		
		this.slice = slice;
		this.skewAngle = skewAngle;
		String title = lsName + ".tif";
		subImage = NewImage.createShortImage(title,lsLength,lsWidth,1,0);
		subImage.copyScale(imp);
		fillSubImage(imp,line,regions,slice,skewAngle);
		
		rawPlotImp = new ImagePlus();
		rawPlotImp = updateRawPlot();
//		rawPlotImp = makeRawPlot(subImage,regions);
		
		if (bgCorrIndex==-1) bgCorrIndex = (int)Prefs.get("LSDDecon.bgModeI",BG_MIN);
		this.bgCorrIndex = bgCorrIndex;
		lsInts = new double[lsLength];
		corrPlotImp = makeCorrPlot(regions,bgCorrIndex);
		
		lsInfo = info;
		
		lswin = new LSwindow(this);
	}

/*	public LSprocessor (LSinfo lsi, LSwindow lswin) {
		this.lsi = lsi;
		this.lswin = lswin;
		
		ImageWindow win = lsi.imp.getWindow();
        canvas = win.getCanvas();
//        canvas.addMouseMotionListener(this);
		updateLineScanRoi();
	}
*/	
/*	public LSprocessor (ImagePlus imp, LSinfo lsi) {
		IJ.showStatus("Starting LSprocessor...");
		// Use the values in the LSinfo object instead of the saved defaults
		this.lsi = lsi;
		linescanID = lsi.linescanID;
		bgModeIndex = lsi.corrcalc;
//		PLOT_W_PERPIXEL = Prefs.get("LSDDecon.pwpp",PLOT_W_PERPIXEL);
		
		this.imp = imp;
		this.imp.setRoi(lsi.line);
		ImageWindow win = imp.getWindow();
        canvas = win.getCanvas();
        canvas.addMouseMotionListener(this);
        manager = lsi.manager;
		if (lsi.bg1w==0 || lsi.rawLSw == 0 || lsi.bg2w == 0) readPreferences();
		lsi.lsWidth = lsi.bg1w + lsi.rawLSw + lsi.bg2w;
		initialize();
		updateLineScanRoi();
	}
*/	
	public void setLineName(Line line) {
		line.setName("test");
	}
	
	public String getLSID(String lineName) {
		String[] parts = Tools.split(lineName,LSDDecon_Manager.nameDelimiter);
		String lsID = parts[parts.length-1];
		return lsID;
	}
	
	public void savePrefs() {
//		System.out.println("LSprocessor.savePrefs: 0");
		for (int i=0; i<regions.length; i++) {
			Prefs.set("LSDDecon.rw"+i,regions[i].rect.height);
			Prefs.set("LSDDecon.rm"+i,regions[i].getViewMode());
		}
		Prefs.set("LSDDecon.bgModeI",bgCorrIndex);
	}
	
	public void fillSubImage(ImagePlus imp, Line line, LSRegion[] regions, int slice, double skewAngle) {
		ImageProcessor ip = imp.getImageStack().getProcessor(slice);
		ip.setCalibrationTable(imp.getCalibration().getCTable());
		double offset = regions[0].rect.height+(regions[1].rect.height-1)/2;
		
		double dx = line.x2d - line.x1d;
		double dy = line.y2d - line.y1d;
		double length = Math.sqrt(dx*dx+dy*dy); // geometric length, non-integral
		double skewD = Math.tan(skewAngle*Math.PI/180); // incremental skew distance		
		double xincL = dx/length; 				// incremental change in x along the length of the linescan
		double yincL = dy/length; 				// incremental change in y along the length of the linescan
		double xincW = yincL - xincL * skewD; 	// incremental change in x across the width of the linescan
		double yincW = -xincL - yincL * skewD; 	// incremental change in y across the width of the linescan
		
        ImageProcessor ipSub = subImage.getProcessor();
		
		// fill in pixVals from original image
		double rx;
		double ry;
		if (lsLength>=1.0) {
			for (int i=0; i<lsWidth; i++) {
				rx = line.x1d - (i-offset)*xincW;	// initial column
				ry = line.y1d - (i-offset)*yincW;	// initial column
				for (int j=0; j<lsLength; j++) {
	        		ipSub.putPixelValue(j,i,ip.getInterpolatedValue(rx,ry));
					rx += xincL;	// move along the line direction
					ry += yincL;	// move along the line direction
				}
			}
		}
		
		ipSub.invertLut();
		ipSub.resetMinAndMax();
		if (lswin!=null) {
			lswin.ic = lswin.cans[0];
//			System.out.println("LSP.fillSubImage: lswin.ic="+lswin.ic.toString());
		}
		subImage.setProcessor(null,ipSub);
	}
	
/*	public ImagePlus makeRawPlot(ImagePlus imp, LSRegion[] regions) {

    	// Initialize x-values
		double[] xvals = new double[lsLength];
		for (int i=0; i<lsLength; i++) xvals[i] = i;

		// Initialize plot size and limits
		PlotWindow.plotWidth = (int)(LSwindow.PLOT_W_PERPIXEL*lsLength);
		Plot rawPlot = new Plot("Raw LineScan Plot","Distance (pixels)","Intensity",new float[] {0},new float[] {0});
		rawPlot.setLimits(0,lsLength-1,0,imp.getProcessor().getMax());

		// calculate y-values
		Color[] plotColors = {Color.yellow,Color.red,Color.yellow};
		for (int i=0; i<regions.length; i++) {
			double[] values = new double[lsLength];
			if (regions[i].mode<0) {
				rawPlot.setColor(plotColors[i]);
				int mode = regions[i].mode;
				int view = 0;
				for (int j=0; j<LSRegion.TYPES.length; j++) {
					if (mode<=LSRegion.TYPES[j]) {
						values = regions[i].calculateIntensities(imp,LSRegion.TYPES[j]);
						rawPlot.addPoints(xvals,values,Plot.LINE);
						view++;
						mode-=LSRegion.TYPES[j];
					}
				}
				if (view==1) regions[i].ints = values;
			}
			else {
				rawPlot.setColor(Color.blue);
				values = regions[i].calculateIntensities(imp,regions[i].mode);
				rawPlot.addPoints(xvals,values,Plot.LINE);
				regions[i].ints = values;
			}
		}
		
		if (lswin!=null) lswin.ic = lswin.cans[1];
        rawPlotImp.setProcessor(null,rawPlot.getProcessor());
        return rawPlotImp;
//		return rawPlot.getImagePlus();
	}
*/	
	public ImagePlus makeCorrPlot(LSRegion[] regions, int index) {
		boolean isValid = true;
		for (int i=0; i<regions.length; i++) {
			if (regions[i].ints == null) isValid = false;
		}
		if (isValid) {
			for (int i=0; i<lsLength; i++) {
				switch (index) {
				case BG_MIN:
					lsInts[i] = regions[1].ints[i]-Math.min(regions[0].ints[i],regions[2].ints[i]);
					break;
				case BG_AVE:
					lsInts[i] = regions[1].ints[i]-(regions[0].ints[i]+regions[2].ints[i])/2;
					break;
				case BG_1:
					lsInts[i] = regions[1].ints[i]-regions[0].ints[i];
					break;
				case BG_2:
				case BG_CONST:
					lsInts[i] = regions[1].ints[i]-regions[2].ints[i];
					break;
				}
			}
		}
		
		xvals = new double [lsLength];
		for (int i=0; i<lsLength; i++) xvals[i] = i;
		
		PlotWindow.plotWidth = (int)(LSwindow.PLOT_W_PERPIXEL*lsLength);
		Plot bgCorrPlot = new Plot("CorrLSPlot", "Distance (pixels)", "intensity", xvals, lsInts);
		double[] minmax = Tools.getMinMax(lsInts);
		bgCorrPlot.setLimits(0,lsLength-1,0,minmax[1]);
		
		if (lswin!=null) lswin.ic = lswin.cans[2];
		return bgCorrPlot.getImagePlus();
	}
	
/*	public void updateLineScanRoi() {
//			IJ.write("Updating all...");
        double dx = lsi.line.x2d-lsi.line.x1d;
        double dy = lsi.line.y2d-lsi.line.y1d;
        lsi.lsLength = (int)Math.round(Math.sqrt(dx*dx+dy*dy));
		lsi.lsAngle = Math.atan2(dy,dx);
		updateSubImage();
	}
*/
/*    public void updateSubImage() {
		ipSub = makeSubregionImage();
		updatePlots();
	}
*/    
    public ImagePlus updateSubImage2() {
		imp.getProcessor().setCalibrationTable(imp.getCalibration().getCTable());
		double dx = linePts[1].x - linePts[0].x;
		double dy = linePts[1].y - linePts[0].y;
//		double dx = line.x2d - line.x1d;
//		double dy = line.y2d - line.y1d;
		double length = Math.sqrt(dx*dx+dy*dy); // geometric length, non-integral
		double skewD = Math.tan(skewAngle*Math.PI/180); // incremental skew distance		
		double xincL = dx/length; 				// incremental change in x along the length of the linescan
		double yincL = dy/length; 				// incremental change in y along the length of the linescan
		double xincW = yincL - xincL * skewD; 	// incremental change in x across the width of the linescan
		double yincW = -xincL - yincL * skewD; 	// incremental change in y across the width of the linescan
        ImagePlus impNew = NewImage.createShortImage("Subregion",lsLength,lsWidth,1,0);
        ImageProcessor ipNew = impNew.getProcessor();
		if (lsLength>=1.0) {
			double offset = regions[0].rect.height+(regions[1].rect.height-1)/2;
			for (int i=0; i<lsWidth; i++) {
				double rx = linePts[0].x - (i-offset)*xincW;	// initial column
				double ry = linePts[0].y - (i-offset)*yincW;	// initial column
//				double rx = line.x1d - (i-offset)*xincW;	// initial column
//				double ry = line.y1d - (i-offset)*yincW;	// initial column
				for (int j=0; j<lsLength; j++) {
	        		ipNew.putPixelValue(j,i,imp.getProcessor().getInterpolatedValue(rx,ry));
					rx += xincL;	// move along the line direction
					ry += yincL;	// move along the line direction
				}
			}
		}
		ipNew.invertLut();
		ipNew.resetMinAndMax();

		lswin.ic = lswin.cans[0];
		subImage.setProcessor(null,ipNew);
		return subImage;
//		ImagePlus imp2 = subImage;
//      imp2.setProcessor(null,ipNew);
//      return imp2;
    }
    
    public ImagePlus updateRawPlot() {
    	ImageProcessor ip = subImage.getProcessor();
    	// Initialize x-values
		xvals = new double[lsLength];
		for (int i=0; i<lsLength; i++) xvals[i] = i;
		
		// Initialize plot size and limits
		PlotWindow.plotWidth = (int)(LSwindow.PLOT_W_PERPIXEL*lsLength);
		Plot rawPlot = new Plot("Raw Linescan Plot","Distance (pixels)","Intensity",ZERO,ZERO);
		rawPlot.setLimits(0,lsLength-1,0,ip.getMax());
		
		// calculate y-values
		Color[] plotColors = {Color.yellow,Color.red,Color.yellow};
		for (int i=0; i<regions.length; i++) {
			double[] values = new double[lsLength]; // Make a new aray of values for each region
			if (regions[i].mode<0) {
				rawPlot.setColor(plotColors[i]);
				int mode = regions[i].mode;
				int view = 0;
				for (int j=0; j<LSRegion.MODES.length; j++) {
					if (mode<=LSRegion.MODES[j]) {
						values = regions[i].calculateIntensities(subImage,LSRegion.MODES[j]);
						rawPlot.addPoints(xvals,values,Plot.LINE);
						view++;
						mode-=LSRegion.MODES[j];
					}
				}
				if (view==1) regions[i].ints = values;  // Point the region intensities to this array of values
			}
			else {
				rawPlot.setColor(Color.blue);
				for (int j=0; j<lsLength; j++) values[j] = regions[i].mode;
				rawPlot.addPoints(xvals,values,Plot.LINE);
				regions[i].ints = values;
			}
		}

		if (lswin!=null) lswin.ic = lswin.cans[1];
        rawPlotImp.setProcessor(null,rawPlot.getProcessor());
        return rawPlotImp;
        
    }
    
    public ImagePlus updateCorrPlot() {
		lsInts = new double[lsLength];
		int[] views = {0,0,0};
		int m;
		for (int i=0; i<regions.length; i++) {
			m = regions[i].mode;
			for (int j=0; j<LSRegion.MODES.length; j++) {
				if (m<=LSRegion.MODES[j]) {
					views[i]++;
					m-=LSRegion.MODES[j];
				}
			}
		}
        double bgConstValue;// = 0;
		if ((views[0] == 1 && views[1] == 1 && views[2] == 1) || (views[1] == 1 && bgCorrIndex == LSprocessor.BG_CONST)) {
			switch (bgCorrIndex) {
			case LSprocessor.BG_MIN:	
				for(int i=0; i<lsLength; i++) lsInts[i] = regions[1].ints[i]-Math.min(regions[0].ints[i],regions[2].ints[i]); 
				break;
			case LSprocessor.BG_AVE:	
				for(int i=0; i<lsLength; i++) lsInts[i] = regions[1].ints[i]-(regions[0].ints[i]+regions[2].ints[i])/2;
				break;
			case LSprocessor.BG_1:	
				for(int i=0; i<lsLength; i++) lsInts[i] = regions[1].ints[i]-regions[0].ints[i]; 
				break;
			case LSprocessor.BG_2:	
				for(int i=0; i<lsLength; i++) lsInts[i] = regions[1].ints[i]-regions[2].ints[i]; 
				break;
			case LSprocessor.BG_CONST:
				bgConstValue = regions[0].mode;
				for(int i=0; i<lsLength; i++) lsInts[i] = regions[1].ints[i]-bgConstValue; 
				break;
			default: lsInts = regions[1].ints; break;
			}
		}
		
		xvals = new double [lsLength];
		for (int i=0; i<lsLength; i++) xvals[i] = i;
		
		PlotWindow.plotWidth = (int)(LSwindow.PLOT_W_PERPIXEL*lsLength);
		Plot corrPlot = new Plot("Corrected LineScan Plot", "Distance (pixels)", "intensity", xvals, lsInts);
		double[] b = Tools.getMinMax(lsInts);
		corrPlot.setLimits(0,lsLength-1,0,b[1]);
		
        lswin.ic = lswin.cans[2];
        corrPlotImp.setProcessor(null,corrPlot.getProcessor());
        return corrPlotImp;
//        ImagePlus impCorrPlot = corrPlotImp;
//        impCorrPlot.setProcessor(null,corrPlot.getProcessor());
//        return impCorrPlot;
    }
    
/*    public void updatePlots() {
        checkPlotWindow();
		ImageProcessor ip2 = makeRawLSPlot(ipSub); // the imageProcessor for the raw profiles
		ImageProcessor ip3 = makeCorrLSplot(lsi.corrLSint);
		
		// Make the Plots for LSwindow
        if (lsi.impSub==null || plotImp==null || plotImp2==null) {
        	// Make the subImage
        	String name = lsi.basename+".tif";
        	lsi.impSub = new ImagePlus(name,ipSub);
        	lswin.cans[0] = new LineScanCanvas(lsi.impSub,regionRects);
        	lsi.impSub.copyScale(lsi.imp);

            plotImp = new ImagePlus("Raw Plot",ip2);	// raw profiles
            lswin.cans[1] = new LineScanCanvas(plotImp);

            plotImp2 = new ImagePlus("Corrected Plot", ip3);	// corrected profile
            lswin.cans[2] = new LineScanCanvas(plotImp2);

//    		ImageWindow.centerNextImage();
        }

        if (IJ.shiftKeyDown()) {
        	lswin.cans[0].overlay=false;
        	IJ.setKeyUp(KeyEvent.VK_SHIFT);
        }
        else {
        	lswin.cans[0].overlay = true;
        }
      	
        lswin.ic = lswin.cans[0];
        lsi.impSub.setProcessor(null,ipSub);
        lswin.cans[0].updateImage(lsi.impSub);

        lswin.ic = lswin.cans[1];
        plotImp.setProcessor(null,ip2);
        lswin.cans[1].updateImage(plotImp);
		
        lswin.ic = lswin.cans[2];
        plotImp2.setProcessor(null,ip3);
        lswin.cans[2].updateImage(plotImp2);
      	
        if (lswin.complete) lswin.updateImagePanel(ip2.getWidth(),ip2.getHeight());
        
        lsi.saveLSpreferences();
        
        // set the linescan into the manager list
    	lsi.manager.setLSinfo(this.lsi);
    }
*/
    
    // END MAIN PROCESSING
    
    
    
    
    
    // START SUB-ROUTINES
    
    /** Returns an ImageProcessor containing the Image subregion defined by the line roi*/
/*    public ImageProcessor makeSubregionImage() {
		double [][] subPixVals = new double [lsi.lsWidth][lsi.lsLength];
		double offset = lsi.bg1w+(lsi.rawLSw-1)/2;
		lsi.imp.getProcessor().setCalibrationTable(lsi.imp.getCalibration().getCTable());
		
		subPixVals = getSubImage(lsi.imp.getProcessor(),lsi.lsLength,lsi.lsWidth,offset,lsi.skewAngle);
		ImageProcessor ip = createSubImage(subPixVals,lsi.lsWidth,lsi.lsLength);
		return ip;
    }
*/    
	/** Returns an array of pixVals from the specified subregion of the specified ImageProcessor.
	 *  The size of the region is (l,w).  Since this is displayed in landscape it is l wide and w height.
	 *  The position of the region is specified by the line from (x1,y1) to (x2,y2) and by the offset in pixels.
	 *  The returned pixVals are determine from the bilinear interpolation of the original image. */
/*	public double [][] getSubImage(ImageProcessor ip, int lsL, int totW, double offset, double skewAngle) {
		double [][] pixVals = new double [totW][lsL];	// output array of pixel data for subregion
		double dx = lsi.line.x2d - lsi.line.x1d;
		double dy = lsi.line.y2d - lsi.line.y1d;
		double length = Math.sqrt(dx*dx+dy*dy); // geometric length, non-integral
		double skewD = Math.tan(skewAngle*Math.PI/180); // incremental skew distance		
		double xincL = dx/length; 				// incremental change in x along the length of the linescan
		double yincL = dy/length; 				// incremental change in y along the length of the linescan
		double xincW = yincL - xincL * skewD; 	// incremental change in x across the width of the linescan
		double yincW = -xincL - yincL * skewD; 	// incremental change in y across the width of the linescan
		
		// x,y positions of the first column in the linescan, i.e. intersecting x1,y1 
		double [] xbase = new double [totW];
		double [] ybase = new double [totW];
		for (int i=0;i<totW; i++) {
			xbase[i] = lsi.line.x1d - (i-offset)*xincW;  // offset x1 position by -(totW-1)/2 to +(totW-1)/2 increments
			ybase[i] = lsi.line.y1d - (i-offset)*yincW;  // offset x1 position by -(totW-1)/2 to +(totW-1)/2 increments
		}
		// fill in pixVals from original image
		if (lsL>=1.0) {
			for (int i=0; i<totW; i++) {
				double rx = xbase[i];	// initial column
				double ry = ybase[i];	// initial column
				for (int j=0; j<lsL; j++) {
						pixVals[i][j] = ip.getInterpolatedValue(rx,ry);
					rx += xincL;	// move along the line direction
					ry += yincL;	// move along the line direction
				}
			}
		}
		return pixVals;
	}
*/
	/** Returns an ImageProcessor containing the array of pixVals. */
/*	public ImageProcessor createSubImage(double[][] pixVals,int w, int h) {
        ImagePlus imp = NewImage.createShortImage("Subregion",h,w,1,0);
        ImageProcessor ip = imp.getProcessor();
        for (int i=0;i<w;i++) {
        	for (int j=0;j<h;j++) {
        		ip.putPixelValue(j,i,pixVals[i][j]);
        	}
        }
		ip.invertLut();
		ip.resetMinAndMax();
        return ip;
	}
*/	
/*	public ImageProcessor makeRawLSPlot(ImageProcessor ip) {
		// Initialize for generating raw profiles
		double [][] profiles = {{},{},{}};//new double [3][];
		int [] views = {0,0,0}; //new int [3]; views[0]=views[1]=views[2]=0;
		double[] xvals = getXvals(0,lsi.lsLength);
		regionRects[0] = new Rectangle(0,0,lsi.lsLength,lsi.bg1w);
		regionRects[1] = new Rectangle(0,lsi.bg1w,lsi.lsLength,lsi.rawLSw);
		regionRects[2] = new Rectangle(0,lsi.bg1w+lsi.rawLSw,lsi.lsLength,lsi.bg2w);

		PlotWindow.plotWidth = (int)(LSwindow.PLOT_W_PERPIXEL*lsi.lsLength);
		Plot t_plot = new Plot("Raw Linescan Plot","Distance (pixels)","Intensity",new float[] {0},new float[] {0});
		t_plot.setLimits(0,lsi.lsLength,0,ip.getMax());

		// generate profile data
		int [] m = {lsi.bg1calc,lsi.rawLScalc,lsi.bg2calc};
		Color[] plotColors = {Color.yellow,Color.red,Color.yellow};
		int[] types = {LSinfo.MAX,LSinfo.MED,LSinfo.AVE,LSinfo.MIN};
		double [] profile = {};
		for (int i=0; i<m.length; i++) {
			t_plot.setColor(plotColors[i]);
			for (int j=0; j<types.length; j++) {
				if (m[i]>=types[j]) {
					profile = getProfile(regionRects[i],ip,types[j]);
					t_plot.addPoints(xvals,profile,Plot.LINE);
					views[i]++;
					m[i]-=types[j];
				}
			}
			if (views[i]==1) profiles[i] = profile;
		}
		
		// Add constant value line
		if (lsi.corrcalc == 5) {
			t_plot.setColor(Color.blue);
			double [] x = {xvals[0],xvals[xvals.length-1]};
			double [] y = {lsi.bgConstValue,lsi.bgConstValue};
			t_plot.addPoints(x,y,Plot.LINE);
		}

		// Automatically update this profile whenever possible
		lsi.corrLSint = new double[lsi.lsLength];
		if ((views[0] == 1 && views[1] == 1 && views[2] == 1) || (views[1] == 1 && lsi.corrcalc == 5)) {
			lsi.corrLSint = calcCorrLS(profiles[0],profiles[1],profiles[2],lsi.corrcalc);	// calculate corrected linescan
		}
		return t_plot.getProcessor();
	}
*/	
/*	public double[] getProfile(Rectangle rect, ImageProcessor ip, int mode) {
		int along = rect.width;
		int across = rect.height;
		double[] values = new double[along];
		double[] samples = new double[across];
		// i,j are respectively indexes along the length of, and across the width of, the linescan
		for (int i=0; i<along; i++) {
			for (int j=0; j<across; j++) {
				samples[j] = ip.getPixelValue(i+rect.x,j+rect.y);
			}
			if (mode==LSinfo.MAX) values[i] = getMaximum(samples);
			else if (mode==LSinfo.MED) values[i] = getMedian(samples);
			else if (mode==LSinfo.AVE) values[i] = getAverage(samples);
			else if (mode==LSinfo.MIN) values[i] = getMinimum(samples);
		}
		return values;
	}
*/
/*	public double getMinimum(double[] vals) {
		double min = vals[0];
		for (int i=1; i<vals.length; i++) min = Math.min(min,vals[i]);
		return min;
	}
*/	
/*	public double getMaximum(double[] vals) {
		double max = 0;
		for (int i=0; i<vals.length; i++) max = Math.max(max,vals[i]);
		return max;
	}
*/	
/*	public double getMedian(double[] vals) {
		// Assuming list is small, the first half is sorted
		int len = vals.length;
		int medInd = (len % 2!=0) ? (len-1)/2 : len/2; //medInd = index of median in the sorted list (or second of 2)
		for (int i=0; i<=medInd;i++) {	// index i is the position looking to be filled in the sorted list
			double min = vals[i];
			int minInd = i;
			for (int j=i+1; j<len; j++) {	// index j is the position being searched to fill the looked-for position
				if (vals[j]<min) {
					minInd = j;	// by the end of the search minInd is the index of the minimum value
					min = vals[j];	// and min, is that value
				}
			}
			// swap vals[minInd] with vals[i]
			double temp = vals[i];
			vals[i] = vals[minInd];
			vals[minInd] = temp;
		}
		double med = (len % 2!=0) ? vals[medInd] : (vals[medInd-1]+vals[medInd])/2;	// take middle or ave middle 2
		return med;
	}
*/	
/*	public double getAverage(double[] vals) {
		double ave = 0;
		double sum = 0;
		for (int i=0; i<vals.length; i++) sum += vals[i];
		ave = sum/vals.length;
		return ave;
	}
*/	
	/** Returns the ImageProcessor for a profile plot.  X & Y-axes start at zero. */
/*	public ImageProcessor makeCorrLSplot(double[] profile) {
		int n = profile.length;
		double[] xValues = new double [n];
		for (int i=0; i<n; i++) xValues[i] = i;
		PlotWindow.plotWidth = (int)(LSwindow.PLOT_W_PERPIXEL*n);
		Plot plot = new Plot("Corrected LineScan Plot", "Distance (pixels)", "intensity", xValues, profile);
	    double[] a = Tools.getMinMax(xValues);
		double[] b = Tools.getMinMax(profile);
		plot.setLimits(a[0],a[1],0,b[1]);
		return plot.getProcessor();
	}
*/	
/*	double [] getXvals(int start, int end) {
		int len = end-start;
		double [] x = new double[len];
		for (int i=0;i<len;i++) x[i] = i+start;
		return x;
	}
*/
/*	double [] calcCorrLS(double[] bg1, double[] rawLS, double[] bg2, int mode) {
		int len = rawLS.length;
		double [] corrLS = new double [len];
		switch (mode) {
			case 0:	for(int i=0; i<len; i++) corrLS[i] = rawLS[i]-Math.min(bg1[i],bg2[i]); break;
			case 1:	for(int i=0; i<len; i++) corrLS[i] = rawLS[i]-(bg1[i]+bg2[i])/2; break;
//				case 2:	corrLS = rawLS; break; //for(int i=0; i<len; i++) corrLS[i] = rawLS[i];
			case 3:	for(int i=0; i<len; i++) corrLS[i] = rawLS[i]-bg1[i]; break;
			case 4:	for(int i=0; i<len; i++) corrLS[i] = rawLS[i]-bg2[i]; break;
			case 5: for(int i=0; i<len; i++) corrLS[i] = rawLS[i]-lsi.bgConstValue; break;
			default: corrLS = rawLS; break;
		}
		return corrLS;
	}
*/	
/*	public boolean[] decodeMode(int region) {
		int mode = 0;
		switch (region) {
		case 0: mode = lsi.bg1calc; break;
		case 1: mode = lsi.rawLScalc; break;
		case 2: mode = lsi.bg2calc; break;
		}
		return new boolean[] {(mode&LSinfo.MAX)!=0,(mode&LSinfo.MED)!=0,(mode&LSinfo.AVE)!=0,(mode&LSinfo.MIN)!=0};
	}
*/	
	String getShortInfo(String info) {
		String s;
		int b = info.indexOf("Extended");
		s = info.substring(0,b);
		return s;
	}
    
    /** returns true if there is a line selection */
    boolean isSelection() {
        if (imp==null)
            return false;
        Roi roi = imp.getRoi();
        if (roi==null)
            return false;
        int roiType = roi.getType();
        if (roiType==Roi.LINE)
            return true;
       else
            return false;
    }

    // stop listening for mouse events if the subregion window has been closed
    void checkPlotWindow() {
        if (subImage==null) 
            return;
       if (lswin==null || lswin.isVisible()) 
            return;
       ImageWindow win = imp.getWindow();
       if (win==null)
           return;
//       stopListening();
//	       IJ.write("Removed listeners");
    }

    // stop listening for mouse events if the subregion window has been closed
    void stopListening() {
//    	System.out.println("LSprocessor.stopListening: start");
    	ImageWindow win = imp.getWindow();
//    	System.out.println("LSprocessor.stopListening: win="+ win+"; canvas="+win.getCanvas());
//    	for (int i=0; i<win.getCanvas().getListeners(LSwindow.class).length; i++) {
//        	System.out.println("LSprocessor.stopListening: 0 listener["+i+"]= "+win.getCanvas().getListeners(LSwindow.class)[i]);
//    		
//    	}
    	win.getCanvas().removeMouseMotionListener(lswin);
    	Scrollbar sb = null;
    	if (win instanceof StackWindow) {
    		StackWindow s = (StackWindow)win;
    		Component[] c = s.getComponents();
    		for (int i=0; i<c.length; i++) {
//    			System.out.println("LSw.<init>: c["+i+"]= "+c.toString());
    			if (c[i] instanceof Scrollbar) sb = (Scrollbar)c[i];
    		}
    		if (sb!=null)sb.removeMouseListener(lswin);
    	}
//    	for (int i=0; i<win.getCanvas().getListeners(LSwindow.class).length; i++) {
//        	System.out.println("LSprocessor.stopListening: 1 listener["+i+"]= "+win.getCanvas().getListeners(LSwindow.class)[i]);
//    		
//    	}
//		canvas = lsi.imp.getWindow().getCanvas();
//		canvas.removeMouseMotionListener(this);
    }

//    public void showLS(String msg) {
//		IJ.showMessage("LineScan", msg);
//	}

/*    public void mouseDragged(MouseEvent e) {
    	lsi.line = (Line)lsi.imp.getRoi();
    	if (lswin != null) lswin.setROIvalues();
        updateLineScanRoi();
    }
*/
//   public void mouseMoved(MouseEvent e) {}
}
