/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.Prefs;
import ij.WindowManager;
import ij.gui.ImageCanvas;
import ij.gui.ImageWindow;
import ij.gui.Line;
import ij.gui.NewImage;
import ij.gui.Plot;
import ij.gui.PlotWindow;
import ij.gui.Roi;
import ij.gui.StackWindow;
import ij.io.FileInfo;
import ij.io.FileSaver;
import ij.io.OpenDialog;
import ij.io.Opener;
import ij.measure.Calibration;
import ij.plugin.frame.PlugInFrame;
import ij.process.ImageProcessor;
import ij.util.Tools;

import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.Scrollbar;
import java.awt.SystemColor;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Date;

public class LSwindow extends PlugInFrame implements ActionListener, ItemListener, MouseListener, MouseMotionListener {
    
	public static final double PLOT_W_PERPIXEL = Prefs.get("LSDDecon.pwpp",4.0);
    LSprocessor ls;
//    LineScanCanvas[] cans;
	
	
	
//	LSDDecon_Manager manager;
//	LSinfo lsi;
    double subScale = PLOT_W_PERPIXEL;
	double SmIncPxls = 0.1;
	double LgIncPxls = 1;
    String information;
    public boolean complete = false;

	// Parts of the display
    private Panel panActs, panLineInfo, panProfile, panAdj;
    private Button butSave, butLoad, butDDecon;
//    private Button butMoveN, butMoveW, butMoveE, butMoveS;
    public Button[] butMove = new Button[4];
    private Button butSkewL, butSkewR;
    private LField lfSkewA;
    public TextArea taInfo;
    Label[] labCP = new Label[2]; // Labels of the line ROI real positions (double values)
	public Label labSRLength, labSRWidth, labSRAngle;
//	private ViewPanel [] region;
    private Choice bgcorrMd;
    private TextField bgcorrConst;
    Checkbox[] cbMoveLine = new Checkbox[2];
    private Checkbox rbRefFrame0, rbRefFrame1;
    private CheckboxGroup mvRefFrame = new CheckboxGroup();
    private boolean isOrigRefFrame = Prefs.get("LSDDecon.orf",false);
    public LField lfincLG,lfincSM;
//    private TextField tfBG1w, tfLSw, tfBG2w;
    public LField[] lfRegW;
    ImageCanvas ic;//,ic1,ic2,ic3;
    LineScanCanvas[] cans = new LineScanCanvas[3];
    Panel masterPanel, panCont, ImagePanel, panSubImage, panRawLS, panCorrLS;
    
    
    private static final int TEXT_GAP = 10;
	private static boolean centerOnScreen;
    private int textGap = centerOnScreen?0:TEXT_GAP;
    private int textoffset = 530;
    
    LSwindow(LSprocessor ls) {
    	super(ls.lsName);
    	this.ls = ls;
    	
    	// Listen to the canvas to update the line
    	ImageWindow win = ls.imp.getWindow();
    	System.out.println("LSwindow.<init>: imp="+ls.imp+"; win="+win);
    	win.getCanvas().addMouseMotionListener(this);
    	
    	// Find the scroll bar in the stackWindow to listen to changes in the slice
    	Scrollbar sb = null;
    	if (win instanceof StackWindow) {
    		StackWindow s = (StackWindow)win;
    		Component[] c = s.getComponents();
    		for (int i=0; i<c.length; i++) {
//    			System.out.println("LSw.<init>: c["+i+"]= "+c.toString());
    			if (c[i] instanceof Scrollbar) sb = (Scrollbar)c[i];
    		}
    		if (sb!=null)sb.addMouseListener(this);
    	}
    	
    	cans = new LineScanCanvas[3];
    	Rectangle[] rects = new Rectangle[3];
    	for (int i=0; i<rects.length; i++) {
    		rects[i] = ls.regions[i].rect;
    	}
    	cans[0] = new LineScanCanvas(ls.subImage,rects);
        cans[1] = new LineScanCanvas(ls.rawPlotImp);
        cans[2] = new LineScanCanvas(ls.corrPlotImp);
    	makeWindow();
        show();
    	updateWindowDimensions();
        WindowManager.addWindow(this);
    }
    
/*    LSwindow(LSinfo lsi) {
    	super("LineScan of "+lsi.imp.getTitle());
    	this.lsi = lsi;
    	this.manager = lsi.manager;
    	
    	// Start the LSprocessor
    	this.ls = new LSprocessor(lsi,this);
    	
    	// Set-up Panels
        int bd = 5;
        int wbd = 20;
        int w1 = 500;
        int w2 = cans[1].getWidth();
		int subH = (int)(cans[0].getHeight()*subScale);
        int ic2h = cans[1].getHeight();
        int wt = w1+w2+6*bd;
        int ht = subH+2*ic2h+8*bd;
        setLayout(null);
		setForeground(SystemColor.controlText);
		setBackground(SystemColor.control);
        setSize(wt+2*wbd,ht+20+2*wbd);
        masterPanel = makeMasterPanel();
        masterPanel.setBounds(wbd,20+wbd,wt,ht);
        add(masterPanel);
        masterPanel.repaint();
        show();
        updateImagePanel(ls.plotImp.getWidth(),ls.plotImp.getHeight());
        complete = true;
        WindowManager.addWindow(this);
    }
*/    
/*    LSwindow(LSprocessor ls, LSinfo lsi) {
    	super("LineScan of "+ls.imp.getTitle());
//    	System.out.println("Start LSwindow Construction...");
        this.ls = ls;
        manager = ls.manager;
        manager.LSwin = this;
        this.lsi = lsi;
        cans = ls.cans;
        int bd = 5;
        int wbd = 20;
        int w1 = 500;
        int w2 = cans[1].getWidth();
		int subH = (int)(cans[0].getHeight()*subScale);
        int ic2h = cans[1].getHeight();
        int wt = w1+w2+6*bd;
        int ht = subH+2*ic2h+8*bd;
        initialize();
        setLayout(null);
		setForeground(SystemColor.controlText);
		setBackground(SystemColor.control);
        setSize(wt+2*wbd,ht+20+2*wbd);
        masterPanel = makeMasterPanel();
        masterPanel.setBounds(wbd,20+wbd,wt,ht);
        add(masterPanel);
        masterPanel.repaint();
        show();
        WindowManager.addWindow(this);
    }
*/
    public void updateImagePanel (int width, int height) {
        int bd = 5;
        int wbd = 20;
        int w1 = 300;
        int w2 = width; //ic2.getWidth();
        int ich = cans[0].getHeight();
        int ic2h = height;//ic2.getHeight();
        int wt = w1+w2+6*bd;
        int ht = ich+2*ic2h+8*bd;

        this.setSize(wt+2*wbd,ht+20+2*wbd);
        masterPanel.setSize(wt,ht);
        panCont.setSize(panCont.getWidth(),ht);
        ImagePanel.setSize(w2+4*bd,ht);
        panSubImage.setSize(w2+2*bd,ich+2*bd);
        panRawLS.setBounds(bd,ich+3*bd,w2+2*bd,ic2h+2*bd);
        panCorrLS.setBounds(bd,ich+ic2h+5*bd,w2+2*bd,ic2h+2*bd);
        cans[0].setBounds(50+bd,bd,w2,ich);
        cans[1].setBounds(bd,bd,w2,ic2h);
        cans[2].setBounds(bd,bd,w2,ic2h);
        cans[1].setDrawingSize(w2,ic2h);
        cans[2].setDrawingSize(w2,ic2h);
        cans[1].repaint();
        cans[2].repaint();
    }
    
	/** Override Container getInsets() to make room for some text above the image. */
	public Insets getInsets() {
		Insets insets = super.getInsets();
		return new Insets(insets.top+TEXT_GAP, insets.left, insets.bottom, insets.right);
	}

    public void drawInfo(Graphics g) {
    }

    public void paint(Graphics g) {}
    
/*    private int[] getRegionWidths() {
    	int [] widths = new int [ls.regions.length];
    	widths[0] = (int)Double.parseDouble(tfBG1w.getText());
    	widths[1] = (int)Double.parseDouble(tfLSw.getText());
    	widths[2] = (int)Double.parseDouble(tfBG2w.getText());
    	return widths;
    }
*/    
/*    private int [] getViewCalcMode() {
    	int [] modes = new int [4];
    	for (int i=0; i<ls.regions.length; i++) {
    		modes[i] = 0;
    		for (int j=0; j<ls.regions[i].views.length; j++) {
    			modes[i] += (ls.regions[i].views[j].getState()?LSRegion.MODES[j]:0);
    		}
    	}
//    	for (int i=0; i<region.length; i++) {
//    		modes[i] = (region[i].views[0].getState()?LSinfo.MAX:0)
//					+ (region[i].views[1].getState()?LSinfo.MED:0)
//					+ (region[i].views[2].getState()?LSinfo.AVE:0) 
//					+ (region[i].views[3].getState()?LSinfo.MIN:0);
//    	}
    	modes[3] = bgcorrMd.getSelectedIndex();
    	return modes;
    }
*/    
    /** set ROI values on the control panel based on directly moving the ROI with the mouse */
/*    public void setROIvalues() {
    	labCP[0].setText("("+IJ.d2s(lsi.line.x1d,2)+","+IJ.d2s(lsi.line.y1d,2)+")");
    	labCP[1].setText("("+IJ.d2s(lsi.line.x2d,2)+","+IJ.d2s(lsi.line.y2d,2)+")");
    	labSRLength.setText(" Total Length = "+lsi.lsLength+" pixels");
    	labSRAngle.setText(" Line Angle = "+IJ.d2s((180*lsi.lsAngle/Math.PI),2)+" degrees");
    }
*/    
    public Point2D.Double[] getCoords(Line line) {
    	Point2D.Double[] coords = new Point2D.Double[2];
    	coords[0] = new Point2D.Double(line.x1d,line.y1d);
    	coords[1] = new Point2D.Double(line.x2d,line.y2d);
    	return coords;
    }
    
/*    public void saveSubImage() {
    	setSubImageInfo();
    	FileSaver fs = new FileSaver(lsi.impSub);
    	fs.save();
    }
*/    
/*    public void setSubImageInfo() {
    	FileInfo fi = lsi.impSub.getFileInfo();
    	fi.directory = lsi.ofi.directory;
    	fi.fileName = lsi.impSub.getTitle();
    	fi.fileFormat = FileInfo.TIFF;
    	fi.description = LSDDecon_Manager.DESCRIPTION;
    	fi.info = "skewAngle=" + lsi.skewAngle
    			+ "; regionWidths=[" + lsi.bg1w
    			+ "," + lsi.rawLSw
    			+ "," + lsi.bg2w
    			+ "]"
    			+ "; line: x1d,y1d=(" + IJ.d2s(lsi.line.x1d,2)
    			+ "," + IJ.d2s(lsi.line.y1d,2)
    			+ "); x2d,y2d=(" + IJ.d2s(lsi.line.x2d,2)
    			+ "," + IJ.d2s(lsi.line.y2d,2)
    			+ ")";
    	lsi.impSub.setFileInfo(fi);
    }
*/    
/*    public void loadSubImage(String name) {
    	String defName = lsi.ofi.fileName + "_" + lsi.slice + "_" + lsi.linescanID + ".tif";
    	String dir = lsi.ofi.directory;
    	if (name.equals("")) {
    		OpenDialog od = new OpenDialog("Open TIFF subimage",dir,defName);
    		dir = od.getDirectory();
    		name = od.getFileName();
    	}
    	Opener o = new Opener();
    	lsi.impSub = o.openTiff(dir,name);
    }
*/    
/*    public void doDDecon() {
    	lsi.saveLSpreferences();
    	manager.doDDecon();
    }
*/    
    public void doChangeRegionWidths() {
//    	System.out.println("LSwin.doChRegWth7");
    	
    	// Read widths from the GUI, update the region widths, and the subImage rectangles
		int widthSum = 0;
		int widthRegion;
    	for (int i=0; i<ls.regions.length; i++) {
    		widthRegion = (int)Double.parseDouble(lfRegW[i].getText());
    		ls.regions[i].rect.setBounds(0,widthSum,ls.lsLength,widthRegion);
    		cans[0].regions[i] = ls.regions[i].rect;
    		widthSum += widthRegion;
    	}
    	ls.lsWidth = widthSum;
    	labSRWidth.setText(" Total Width = " + ls.lsWidth + " pixels");
    	
    	// Update the subimage
    	ls.subImage = ls.updateSubImage2();
        this.cans[0].updateImage(ls.subImage);
		
		// Update the raw linescan plot
        ls.rawPlotImp = ls.updateRawPlot();
        cans[1].updateImage(ls.rawPlotImp);
		
        // Update the corrected linescan plot
        ls.corrPlotImp = ls.updateCorrPlot();
        cans[2].updateImage(ls.corrPlotImp);
        
        // Update the window dimensions
        updateWindowDimensions();
    }
    
    public void updateWindowDimensions() {
        int bd = 5;
        int wbd = 20;
        int w1 = 300;
        int w2 = ls.rawPlotImp.getProcessor().getWidth();
        int ich = cans[0].getHeight();
        int ic2h = ls.rawPlotImp.getProcessor().getHeight();
        int wt = w1+w2+6*bd;
        int ht = ich+2*ic2h+8*bd;
        this.setSize(wt+2*wbd,ht+20+2*wbd);
        masterPanel.setSize(wt,ht);
        panCont.setSize(panCont.getWidth(),ht);
        ImagePanel.setSize(w2+4*bd,ht);
        panSubImage.setSize(w2+2*bd,ich+2*bd);
        panRawLS.setBounds(bd,ich+3*bd,w2+2*bd,ic2h+2*bd);
        panCorrLS.setBounds(bd,ich+ic2h+5*bd,w2+2*bd,ic2h+2*bd);
        cans[0].setBounds(50+bd,bd,w2,ich);
        cans[1].setBounds(bd,bd,w2,ic2h);
        cans[2].setBounds(bd,bd,w2,ic2h);
        cans[1].setDrawingSize(w2,ic2h);
        cans[2].setDrawingSize(w2,ic2h);
        cans[1].repaint();
        cans[2].repaint();
    }
    
/*    public void doChangeRegionWidths2() {
    	int[] wRegions = getRegionWidths(); // From TextFields
    	lsi.bg1w = wRegions[0];
    	lsi.rawLSw = wRegions[1];
    	lsi.bg2w = wRegions[2];
    	lsi.lsWidth = wRegions[0]+wRegions[1]+wRegions[2];
    	labSRWidth.setText(" Total Width = " + lsi.lsWidth + " pixels");
    	ls.updateSubImage();
    }
*/    
/*    public void doChangeSkewAngle(double delta) {
    	lsi.skewAngle = Double.parseDouble(lfSkewA.getText());
    	lsi.skewAngle += delta;
    	if (delta!=0)lfSkewA.setText(IJ.d2s(lsi.skewAngle));
    	ls.updateSubImage();
    }
*/    
    public void doChangeSkewAngle2(double delta) {
    	ls.skewAngle = Double.parseDouble(lfSkewA.getText());
    	ls.skewAngle += delta;
    	if (delta!=0)lfSkewA.setText(IJ.d2s(ls.skewAngle));

    	// Update the subimage
    	ls.subImage = ls.updateSubImage2();
        this.cans[0].updateImage(ls.subImage);

		// Update the raw linescan plot
        ls.rawPlotImp = ls.updateRawPlot();
        cans[1].updateImage(ls.rawPlotImp);

        // Update the corrected linescan plot
        ls.corrPlotImp = ls.updateCorrPlot();
        cans[2].updateImage(ls.corrPlotImp);

        // Update the window dimensions
        updateWindowDimensions();
    }
    
    public void doChangeBgConst(int value) {
//    	System.out.println("doChangeBgConst");
    	if (ls.bgCorrIndex == LSprocessor.BG_CONST) {
    		ls.regions[0].mode = value;
    		ls.regions[2].mode = value;
    	}
    	
		// Update the raw linescan plot
        ls.rawPlotImp = ls.updateRawPlot();
        cans[1].updateImage(ls.rawPlotImp);

        // Update the corrected linescan plot
        ls.corrPlotImp = ls.updateCorrPlot();
        cans[2].updateImage(ls.corrPlotImp);

        // Update the window dimensions
        updateWindowDimensions();
    }
    

    /** move line according to the adjustment */
/*    public void doMoveLine(boolean isShift, int NS, int EW) { // was moveROI()
    	boolean[] isActive = new boolean[2];
    	isActive[0] = cbMoveLine[0].getState();
    	isActive[1] = cbMoveLine[1].getState();
    	double inc = isShift?SmIncPxls:LgIncPxls;
    	
    	Point2D.Double mv = new Point2D.Double(EW,NS);
    	if (!rbRefFrame0.getState()) {
        	double mc = Math.cos(lsi.lsAngle);
        	double ms = Math.sin(lsi.lsAngle);
    		mv.setLocation((EW*mc-NS*ms),(EW*ms+NS*mc));
    	}
    	
    	Point2D.Double[] ends = getCoords(lsi.line);
    	for (int i=0; i<ends.length; i++) {
    		if (isActive[i]) {
    			ends[i].x += inc * mv.x;
    			ends[i].y += inc * mv.y;
        		this.labCP[i].setText("("+IJ.d2s(ends[i].x,2)+","+IJ.d2s(ends[i].y,2)+")");
    		}
    	}
    	lsi.line.x1d = ends[0].x;
    	lsi.line.x2d = ends[1].x;
    	lsi.line.y1d = ends[0].y;
    	lsi.line.y2d = ends[1].y;
    	lsi.line.x1 = (int)ends[0].x;
    	lsi.line.x2 = (int)ends[1].x;
    	lsi.line.y1 = (int)ends[0].y;
    	lsi.line.y2 = (int)ends[1].y;
    	
		lsi.imp.setRoi(lsi.line);
//		double dx = lsi.line.x2d-lsi.line.x1d;
//		double dy = lsi.line.y2d-lsi.line.y1d;
//		lsi.lsLength = (int)Math.round(Math.sqrt(dx*dx+dy*dy));
//		lsi.lsAngle = Math.atan2(dy,dx);
    	ls.updateLineScanRoi();
    	labSRLength.setText(" Total Length = " + lsi.lsLength + " pixels");
    	labSRAngle.setText(" Line Angle = " + IJ.d2s((180*lsi.lsAngle/Math.PI),2) + " degrees");
    }
*/    
    /** move line according to the adjustment */
    public void doMoveLine2(int NS, int EW, boolean isShift) { // was doMoveLine()
    	// Get state of checkboxes for endpoints to determine which one(s) to move
    	boolean[] isActive = new boolean[2];
    	isActive[0] = cbMoveLine[0].getState(); // true if moving 1st point
    	isActive[1] = cbMoveLine[1].getState(); // true if moving 2nd point
    	if (!isActive[0] && !isActive[1]) return;
    	
    	// Get the increment (in pixels) to move
    	SmIncPxls = Double.parseDouble(lfincSM.getText());
    	LgIncPxls = Double.parseDouble(lfincLG.getText());
    	double inc = isShift?SmIncPxls:LgIncPxls; // true if moving by the small increment
    	if (inc==0) return;
    	
    	// Determine which reference frame to move in
    	Point2D.Double mv = new Point2D.Double();
    	if (rbRefFrame0.getState()) {
    		mv.setLocation(EW,NS);
    	}
    	else {
        	double mc = Math.cos(ls.lsAngle);
        	double ms = Math.sin(ls.lsAngle);
    		mv.setLocation((EW*mc-NS*ms),(EW*ms+NS*mc));
    	}
//    	System.out.println("LSwin.doMoveLine2: mv=("+mv.x+","+mv.y+")");
    	    	
    	// Move the endpoints of the line
    	Point2D.Double[] ends = ls.linePts;
//    	Point2D.Double[] ends = getCoords(ls.line);
//    	System.out.println("LSwin.doMoveLine2: ends[0]=("+ends[0].x+","+ends[0].y+")");
//    	System.out.println("LSwin.doMoveLine2: ends[1]=("+ends[1].x+","+ends[1].y+")");
    	for (int i=0; i<ends.length; i++) {
    		if (isActive[i]) {
    			ends[i].x += inc * mv.x;
    			ends[i].y += inc * mv.y;
//    	    	System.out.println("LSwin.doMoveLine2: ends["+i+"]=("+ends[i].x+","+ends[i].y+")");
    			
    			// Update the display of the endpoints for the line
        		this.labCP[i].setText("("+IJ.d2s(ends[i].x,2)+","+IJ.d2s(ends[i].y,2)+")");
    		}
    	}
    	// Update the line coordinates
    	updateLineOnImage(ends);
//    	updateLineOnImage(ls.line,ends);
//		ls.imp.setRoi(ls.line);
//		ls.imp.draw();

		// Update linescan angle and length
        double dx = ls.linePts[1].x-ls.linePts[0].x;
        double dy = ls.linePts[1].y-ls.linePts[0].y;
//        double dx = ls.line.x2d-ls.line.x1d;
//        double dy = ls.line.y2d-ls.line.y1d;
		ls.lsAngle = Math.atan2(dy,dx);
        ls.lsLength = (int)Math.round(Math.sqrt(dx*dx+dy*dy));
        for (int i=0; i<ls.regions.length; i++) {
        	ls.regions[i].rect.width = ls.lsLength;
        }
    	labSRLength.setText(" Total Length = " + ls.lsLength + " pixels");
    	labSRAngle.setText(" Line Angle = " + IJ.d2s((180*ls.lsAngle/Math.PI),2) + " degrees");

    	// Update the subimage
    	ls.subImage = ls.updateSubImage2();
        this.cans[0].updateImage(ls.subImage);
		
		// Update the raw linescan plot
        ls.rawPlotImp = ls.updateRawPlot();
        cans[1].updateImage(ls.rawPlotImp);

        // Update the corrected linescan plot
        ls.corrPlotImp = ls.updateCorrPlot();
        cans[2].updateImage(ls.corrPlotImp);
        
        // Update the window dimensions
        updateWindowDimensions();
    }
    
    public void updateLineOnImage(Point2D.Double[] e) {
    	Line ol = (Line)ls.imp.getRoi();
    	double olw = Math.abs(ol.x1d-ol.x2d);
    	double olh = Math.abs(ol.y1d-ol.y2d);
    	double m = ls.imp.getWindow().getCanvas().getMagnification();
//    	double tx = ((int)(m*Math.min(e[0].x,e[1].x)))/m;
//    	double ty = ((int)(m*Math.min(e[0].y,e[1].y)))/m;
    	double tw = ((int)(m*Math.abs(e[0].x-e[1].x)))/m;
    	double th = ((int)(m*Math.abs(e[0].y-e[1].y)))/m;
    	double dw = tw - olw;
    	double dh = th - olh;
    	int sw = (int)(dw*m);
    	int sh = (int)(dh*m);
    	System.out.println("LSwin.updateLineOnImage: dw="+dw+"; dh="+dh);
    	System.out.println("LSwin.updateLineOnImage: sw="+sw+"; sh="+sh);
    	int tx = (int)Math.min(e[0].x,e[1].x);
    	int ty = (int)Math.min(e[0].y,e[1].y);
//    	int tw = (int)Math.abs(e[0].x-e[1].x);
//    	int th = (int)Math.abs(e[0].y-e[1].y);
    	if (Math.abs(sw)!=0) {
    		int key = ((sw<0)^(e[0].x<e[1].x))?KeyEvent.VK_RIGHT:KeyEvent.VK_LEFT;
    		for (int i=0; i<Math.abs(sw); i++) {
    			ol.nudgeCorner(key);
    			System.out.println("LSwin.upateLineOnImage: key="+key);
    		}
    	}
/*    	if (sw>0) {
    		int key = (e[0].x<e[1].x)?KeyEvent.VK_RIGHT:KeyEvent.VK_LEFT;
    		for (int i=0; i<sw; i++) {
    			ol.nudgeCorner(key);
    		}
    	}
    	else if (sw<0) {
    		int key = (e[0].x>e[1].x)?KeyEvent.VK_RIGHT:KeyEvent.VK_LEFT;
    		for (int i=0; i<-sw; i++) {
    			ol.nudgeCorner(key);
    		}
    	}
*/    	
    	if (Math.abs(sh)!=0) {
    		int key = ((sh<0)^(e[0].y<e[1].y))?KeyEvent.VK_DOWN:KeyEvent.VK_UP;
    		for (int i=0; i<Math.abs(sh); i++) {
    			ol.nudgeCorner(key);
    			System.out.println("LSwin.upateLineOnImage: key="+key);
    		}
    	}
/*    	if (sh>0) {
    		int key = (e[0].y<e[1].y)?KeyEvent.VK_DOWN:KeyEvent.VK_UP;
    		for (int i=0; i<sh; i++) {
    			ol.nudgeCorner(key);
    		}
    	}
    	else if (sh<0) {
    		int key = (e[0].y>e[1].y)?KeyEvent.VK_DOWN:KeyEvent.VK_UP;
    		for (int i=0; i<-sh; i++) {
    			ol.nudgeCorner(key);
    		}
    	}
*/
    	Rectangle cbr = ol.getBounds();
    	System.out.println("LSwin.updateLineOnImage: cbr="+cbr.toString());
/*    	System.out.println("LSwin.updateLineOnImage: cbr.w="+cbr.width);
    	System.out.println("LSwin.updateLineOnImage:    tw-1/m="+(tw-1/m));
    	System.out.println("LSwin.updateLineOnImage: cbr.w<=(tw-1/m)="+(cbr.width<=(tw-1/m)));
    	while (((cbr.width<=(tw-1/m))&&(e[1].x<e[0].x))||((cbr.width>=(tw+1/m))&&(e[1].x>e[0].x))) {
    		ol.nudgeCorner(KeyEvent.VK_LEFT);
    		cbr = ol.getBounds();
        	System.out.println("LSwin.updateLineOnImage:L cbr.w="+cbr.width);
    	}
    	while (((cbr.width>=(tw+1/m))&&(e[1].x<e[0].x))||((cbr.width<=(tw-1/m))&&(e[1].x>e[0].x))) {
    		ol.nudgeCorner(KeyEvent.VK_RIGHT);
    		cbr = ol.getBounds();
        	System.out.println("LSwin.updateLineOnImage:R cbr.w="+cbr.width);
    	}
    	System.out.println("LSwin.updateLineOnImage: cbr.h="+cbr.height);
    	System.out.println("LSwin.updateLineOnImage:    th-1/m="+(th-1/m));
    	System.out.println("LSwin.updateLineOnImage: cbr.h<=(th-1/m)="+(cbr.height<=(th-1/m)));
    	while (((cbr.height<=(th-1/m))&&(e[1].y<e[0].y))||((cbr.height>=(th+1/m))&&(e[1].y>e[0].y))) {
    		ol.nudgeCorner(KeyEvent.VK_UP);
    		cbr = ol.getBounds();
        	System.out.println("LSwin.updateLineOnImage:U cbr.h="+cbr.height);
    	}
    	while (((cbr.height>=(th+1/m))&&(e[1].y<e[0].y))||((cbr.height<=(th-1/m))&&(e[1].y>e[0].y))) {
    		ol.nudgeCorner(KeyEvent.VK_DOWN);
    		cbr = ol.getBounds();
        	System.out.println("LSwin.updateLineOnImage:D cbr.h="+cbr.height);
    	}
*/
    	ol.setLocation(tx,ty);
    	ol.x1 = (int)e[0].x;
    	ol.x2 = (int)e[1].x;
    	ol.y1 = (int)e[0].y;
    	ol.y2 = (int)e[1].y;
    	
    	ol.x1d = e[0].x;
    	ol.x2d = e[1].x;
    	ol.y1d = e[0].y;
    	ol.y2d = e[1].y;

    	ls.imp.setRoi(ol);
    	
    	ls.manager.updateLR(ol);
    }
    
    public void actionPerformed(ActionEvent e) {
        Object a = e.getSource();
        for (int i=0; i<ls.regions.length; i++) {
        	if (a==lfRegW[i]) {
        		doChangeRegionWidths();
//        		return;
        	}
        }
        for (int i=0; i<butMove.length; i++) {
        	if (a==butMove[i]) {
            	int NS = (a==butMove[2]?1:0) + (a==butMove[0]?-1:0);
            	int EW = (a==butMove[1]?1:0) + (a==butMove[3]?-1:0);
//            	doMoveLine(IJ.shiftKeyDown(), NS, EW); // XXX Not working with IJ.shiftKeyDown
            	doMoveLine2(NS, EW, false);
//            	return;
        	}
        }
//        if (a==butSave) {
//        	saveSubImage();
//        }
//        else if (a==butLoad) {
//        	loadSubImage("");
//        }
        if (a==butDDecon) {
        	ls.manager.doDDecon();
//        else if (a==butDDecon) {
//        	doDDecon();
        }
/*      else if (a==tfBG1w || a==tfLSw || a==tfBG2w) {
        	doChangeRegionWidths2();
      }
*/
        if (a==butSkewL || a==butSkewR || a==lfSkewA) {
        	double delta = ((a==butSkewL)?-5:0) + ((a==butSkewR)?5:0);
        	doChangeSkewAngle2(delta);
//        	doChangeSkewAngle(delta);
        }
//		else if (a==butMoveN || a==butMoveW || a==butMoveE || a==butMoveS) {
//        	int NS = ((a==butMoveS)?1:0) + ((a==butMoveN)?-1:0);
//        	int EW = ((a==butMoveE)?1:0) + ((a==butMoveW)?-1:0);
//        	doMoveLine(IJ.shiftKeyDown(), NS, EW); // XXX Not working with IJ.shiftKeyDown
//		}
        
//        else if (a==lfincSM) SmIncPxls = Double.parseDouble(lfincSM.getText());
//        else if (a==lfincLG) LgIncPxls = Double.parseDouble(lfincLG.getText());
        else if (a==bgcorrConst) {
//        	System.out.println("LSw.actionPerformed: a="+a.toString());
        	int value = (int)Math.round(Double.parseDouble(bgcorrConst.getText()));
        	bgcorrConst.setText(IJ.d2s(value,0));
        	doChangeBgConst(value);
//        	lsi.bgConstValue = (Double.parseDouble(bgcorrConst.getText()));
//        	ls.updatePlots();
        }
        else if (a==taInfo) {
        	ls.lsInfo = taInfo.getText();
        }
        ls.manager.updateLR();
    }
    
    public void itemStateChanged(ItemEvent e) {
        Object source = e.getSource();
        if (source==rbRefFrame0 || source==rbRefFrame1) {
        	Prefs.set("LSDDecon.orf",rbRefFrame0.getState());
        	return;
        }
        if (source==this.bgcorrMd) { // The bg correction mode was changed
        	ls.bgCorrIndex = bgcorrMd.getSelectedIndex();
        	if (ls.bgCorrIndex == LSprocessor.BG_CONST) { // Constant bg mode
        		
        		// activate text field for constant background
        		bgcorrConst.setEnabled(true);
        		
        		// get the previous value within the field
        		int value = (int)Math.round(Double.parseDouble(bgcorrConst.getText()));
        		doChangeBgConst(value);
        		return;
        	}
        	else {
        		bgcorrConst.setEnabled(false);
        		ls.regions[0].mode = ls.regions[0].getViewMode();
        		ls.regions[2].mode = ls.regions[2].getViewMode();
        		
        		// Update the raw linescan plot
                ls.rawPlotImp = ls.updateRawPlot();
                cans[1].updateImage(ls.rawPlotImp);

                // Update the corrected linescan plot
                ls.corrPlotImp = ls.updateCorrPlot();
                cans[2].updateImage(ls.corrPlotImp);

                // Update the window dimensions
                updateWindowDimensions();
                return;
        	}
        }
        else { // One of the region views was changed
        	// get the raw linescan region view
			ls.regions[1].mode = ls.regions[1].getViewMode();
			
			// get the bg region views
	    	if (ls.bgCorrIndex != LSprocessor.BG_CONST) {
	    		ls.regions[0].mode = ls.regions[0].getViewMode();
	    		ls.regions[2].mode = ls.regions[2].getViewMode();
	    	}
	    	
			// Update the raw linescan plot
	        ls.rawPlotImp = ls.updateRawPlot();
	        cans[1].updateImage(ls.rawPlotImp);
	
	        // Update the corrected linescan plot
	        ls.corrPlotImp = ls.updateCorrPlot();
	        cans[2].updateImage(ls.corrPlotImp);
	
	        // Update the window dimensions
	        updateWindowDimensions();
	        return;
        }
    }
    
    public void mouseDragged(MouseEvent e) {
    	Line l = (Line)ls.imp.getRoi();
    	ls.linePts[0].x = l.x1d;
    	ls.linePts[0].y = l.y1d;
    	ls.linePts[1].x = l.x2d;
    	ls.linePts[1].y = l.y2d;
//    	ls.line = (Line)ls.imp.getRoi();
    	labCP[0].setText("("+IJ.d2s(ls.linePts[0].x,2)+","+IJ.d2s(ls.linePts[0].y,2)+")");
    	labCP[1].setText("("+IJ.d2s(ls.linePts[1].x,2)+","+IJ.d2s(ls.linePts[1].y,2)+")");
//    	labCP[0].setText("("+IJ.d2s(ls.line.x1d,2)+","+IJ.d2s(ls.line.y1d,2)+")");
//    	labCP[1].setText("("+IJ.d2s(ls.line.x2d,2)+","+IJ.d2s(ls.line.y2d,2)+")");

		// Update linescan angle, length, and regions
        double dx = ls.linePts[1].x-ls.linePts[0].x;
        double dy = ls.linePts[1].y-ls.linePts[0].y;
//        double dx = ls.line.x2d-ls.line.x1d;
//        double dy = ls.line.y2d-ls.line.y1d;
		ls.lsAngle = Math.atan2(dy,dx);
        ls.lsLength = (int)Math.round(Math.sqrt(dx*dx+dy*dy));
        for (int i=0; i<ls.regions.length; i++) {
        	ls.regions[i].rect.width = ls.lsLength;
        }
    	labSRLength.setText(" Total Length = " + ls.lsLength + " pixels");
    	labSRAngle.setText(" Line Angle = " + IJ.d2s((180*ls.lsAngle/Math.PI),2) + " degrees");

    	// Update the subimage
    	ls.subImage = ls.updateSubImage2();
        this.cans[0].updateImage(ls.subImage);
		
		// Update the raw linescan plot
        ls.rawPlotImp = ls.updateRawPlot();
        cans[1].updateImage(ls.rawPlotImp);
		
        // Update the corrected linescan plot
        ls.corrPlotImp = ls.updateCorrPlot();
        cans[2].updateImage(ls.corrPlotImp);
        
        // Update the window dimensions
        updateWindowDimensions();

    	
//    	lsi.line = (Line)lsi.imp.getRoi();
//    	labCP[0].setText("("+IJ.d2s(lsi.line.x1d,2)+","+IJ.d2s(lsi.line.y1d,2)+")");
//    	labCP[1].setText("("+IJ.d2s(lsi.line.x2d,2)+","+IJ.d2s(lsi.line.y2d,2)+")");
//      ls.updateLineScanRoi();
    }
    
    public void mouseMoved(MouseEvent e) {
//    	Object source = e.getSource();
//		System.out.println("LSw.mouseMoved: source="+source.toString());
    }
    
    public void mousePressed(MouseEvent e) {
//    	Object source = e.getSource();
//		System.out.println("LSw.mousePressed: source="+source.toString());
    }

    public void mouseClicked(MouseEvent e) {
//    	Object source = e.getSource();
//		System.out.println("LSw.mouseClicked: source="+source.toString());
    	int slice = ls.imp.getCurrentSlice();
    	if (slice != ls.slice) {
    		ls.slice = slice;
    		
        	// Update the subimage
        	ls.subImage = ls.updateSubImage2();
            this.cans[0].updateImage(ls.subImage);
    		
    		// Update the raw linescan plot
            ls.rawPlotImp = ls.updateRawPlot();
            cans[1].updateImage(ls.rawPlotImp);
    		
            // Update the corrected linescan plot
            ls.corrPlotImp = ls.updateCorrPlot();
            cans[2].updateImage(ls.corrPlotImp);
            
            // Update the window dimensions
            updateWindowDimensions();
    	}
    }

    public void mouseReleased(MouseEvent e) {
//    	Object source = e.getSource();
//		System.out.println("LSw.mouseReleased: source="+source.toString());
    }

    public void mouseExited(MouseEvent e) {
//    	Object source = e.getSource();
//		System.out.println("LSw.mouseExitked: source="+source.toString());
    }
    
    public void mouseEntered(MouseEvent e) {
//    	Object source = e.getSource();
//		System.out.println("LSw.mouseEntered: source="+source.toString());
    }
    
    public void windowClosed(WindowEvent e) {
    	Prefs.set("LSDDecon.pwpp",LSwindow.PLOT_W_PERPIXEL);
    	ls.savePrefs();
    	ls.stopListening();
    }


    // MAKE PANELS FOR THE WINDOW
    
    private void makeWindow() {
    	
    	// Make all the panels
        masterPanel = new Panel();
	        panCont = new Panel();
		    	Panel panSR = new Panel();
			    	Panel panSRw = new Panel();
			    	Panel panSRi = new Panel();
	        	panLineInfo = new Panel();
			    	Panel panCP = new Panel();
			    	Panel panRF = new Panel();
		    	panAdj = new Panel();
		    		Panel panScr = new Panel();
		    			Panel panScrBut = new Panel();
		    	panProfile = new Panel();
			    	Panel panView = new Panel();
			    	Panel panCalc = new Panel();
		    	panActs = new Panel();
	        ImagePanel = new Panel();
		        panSubImage = new Panel(); // Subimage
		        panRawLS = new Panel(); // Raw Linescans
		        panCorrLS = new Panel(); // Corrected Linescan

		// Set their layouts
        this.setLayout(null);
        	masterPanel.setLayout(null);
        		panCont.setLayout(null);
        			panSR.setLayout(null);
        				panSRw.setLayout(null);
        				panSRi.setLayout(null);
        			panLineInfo.setLayout(null);
        				panCP.setLayout(null);
        				panRF.setLayout(null);
        			panAdj.setLayout(null);
        				panScr.setLayout(null);
        					panScrBut.setLayout(null);
        			panProfile.setLayout(null);
        				panView.setLayout(null);
        				panCalc.setLayout(null);
        			panActs.setLayout(null);
        		ImagePanel.setLayout(null);

        		
        // Set their colors
		this.setForeground(SystemColor.controlText);
		this.setBackground(SystemColor.control);
			masterPanel.setBackground(Color.darkGray);
				panCont.setBackground(Color.gray);
					panSR.setBackground(Color.gray);
					panLineInfo.setBackground(Color.lightGray);
					panAdj.setBackground(Color.gray);
					panProfile.setBackground(Color.lightGray);
					panActs.setBackground(Color.gray);
				ImagePanel.setBackground(Color.gray);
					panSubImage.setBackground(Color.darkGray);
					panRawLS.setBackground(Color.darkGray);
					panCorrLS.setBackground(Color.darkGray);

        
        // Set their bounds
    	int bd = 5;
        int wbd = 20;
        int w1 = 300;
		int subH = (int)(cans[0].getHeight()*subScale);
		int subW = (int)(cans[0].getWidth()*subScale);
        int ic2h = cans[1].getHeight();
        int w2 = cans[1].getWidth();
        int ht = subH+2*ic2h+8*bd;
        int wta = 300+w2+6*bd;
        this.setBounds(0,0,wta+2*wbd,ht+20+2*wbd);
        	masterPanel.setBounds(wbd,20+wbd,wta,ht);
        		panCont.setBounds(0,0,w1+2*bd,ht);
        			panSR.setBounds(bd,bd,w1,100);
        				panSRw.setBounds(0,0,100,100);
        				panSRi.setBounds(120,0,180,100);
        			panLineInfo.setBounds(bd,bd+100,w1,100);
        				panCP.setBounds(140,0,50,100);
        				panRF.setBounds(200,0,100,100);
        			panAdj.setBounds(bd,bd+200,w1,100);
        				panScr.setBounds(0,0,300,100);
        					panScrBut.setBounds(200,5,90,90);
        			panProfile.setBounds(bd,bd+300,w1,170);
        				panView.setBounds(0,0,300,120);
        				panCalc.setBounds(0,120,300,50);
        			panActs.setBounds(bd,bd+590,w1,50);
		        ImagePanel.setBounds(w1+2*bd,0,w2+4*bd,ht);
			        panSubImage.setBounds(bd,bd,w2+2*bd,subH+2*bd);
			        	cans[0].setBounds(50+bd,bd,subW,subH);
				        cans[0].setDrawingSize(subW,subH);
				        cans[0].setMagnification(subScale);
//				        cans[0].setLocation(50+bd,bd);
			        panRawLS.setBounds(bd,subH+3*bd,w2+2*bd,ic2h+2*bd);
			        	cans[1].setLocation(bd,bd);
			        panCorrLS.setBounds(bd,ic2h+subH+5*bd,w2+2*bd,ic2h+2*bd);
			        	cans[2].setLocation(bd,bd);
        
			        	
			        	
    	// Make all of the components
    	Label desc = new Label("Width");
    	desc.setBounds(50,0,50,20);
    	
    	String[] regNames = {"BG1","LSraw","BG2"};
    	lfRegW = new LField[ls.regions.length];
    	for (int i=0; i<ls.regions.length; i++) {
    		ls.regions[i].name = regNames[i];
    		lfRegW[i] = new LField(regNames[i],LField.WEST);
    		lfRegW[i].setColumns(2);
    		lfRegW[i].setBounds(0,20+20*i,100,20,0.5);
    		lfRegW[i].setText(IJ.d2s(ls.regions[i].rect.height,0));
    		lfRegW[i].addActionListener(this);
    	}

    	labSRLength = new Label(" Total Length = "+ls.lsLength+" pixels");
    	labSRLength.setBounds(0,20,180,20);
    	
    	labSRWidth = new Label(" Total Width = "+ls.lsWidth+" pixels");
    	labSRWidth.setBounds(0,40,180,20);
    	
    	labSRAngle = new Label(" Line Angle = "+IJ.d2s((180*ls.lsAngle/Math.PI),2)+String.valueOf(IJ.degreeSymbol));
    	labSRAngle.setBounds(0,60,180,20);

        Label labA = new Label("ROI");
    	labA.setBounds(0,20,40,20);

    	Label labD = new Label("Start");
    	labD.setBounds(0,40,40,20);

    	Label labE = new Label("End");
    	labE.setBounds(0,60,40,20);

    	labCP[0] = new Label("("+IJ.d2s(ls.linePts[0].x,2)+","+IJ.d2s(ls.linePts[0].y,2)+")");
//    	labCP[0] = new Label("("+IJ.d2s(ls.line.x1d,2)+","+IJ.d2s(ls.line.y1d,2)+")");
		labCP[0].setBounds(40,40,100,20);

		labCP[1] = new Label("("+IJ.d2s(ls.linePts[1].x,2)+","+IJ.d2s(ls.linePts[1].y,2)+")");
//		labCP[1] = new Label("("+IJ.d2s(ls.line.x2d,2)+","+IJ.d2s(ls.line.y2d,2)+")");
		labCP[1].setBounds(40,60,100,20);
		
    	Label labCPx = new Label("-- Pts --");
    	labCPx.setBounds(0,20,50,20);

    	cbMoveLine[0] = new Checkbox(" 1 ",null,true);
		cbMoveLine[0].setBounds(0,40,50,20);

		cbMoveLine[1] = new Checkbox(" 2 ",null,true);
		cbMoveLine[1].setBounds(0,60,50,20);
    	
    	Label labRF = new Label("-- Ref Frame --");
    	labRF.setBounds(0,20,100,20);

    	rbRefFrame0 = new Checkbox(" Original ",mvRefFrame,isOrigRefFrame);
    	rbRefFrame0.setBounds(0,40,100,20);
    	rbRefFrame0.addItemListener(this);

    	rbRefFrame1 = new Checkbox(" Subregion ",mvRefFrame,!isOrigRefFrame);
    	rbRefFrame1.setBounds(0,60,100,20);
    	rbRefFrame1.addItemListener(this);
        
    	lfSkewA = new LField("Skew",LField.NORTH);
    	lfSkewA.setColumns(2);
    	lfSkewA.setBounds(5,5,70,40,0.6);
    	lfSkewA.setText(IJ.d2s(ls.skewAngle));
    	lfSkewA.addActionListener(this);

    	butSkewL = new Button("L");
    	butSkewL.setBounds(5,50,30,30);
    	butSkewL.addActionListener(this);

    	butSkewR = new Button("R");
    	butSkewR.setBounds(45,50,30,30);
    	butSkewR.addActionListener(this);

    	lfincLG = new LField("LG inc",LField.WEST);
    	lfincLG.setColumns(2);
    	lfincLG.setBounds(90,20,100,20,0.5);
    	lfincLG.setText(IJ.d2s(LgIncPxls,0));
    	lfincLG.addActionListener(this);

    	lfincSM = new LField("SM inc",LField.WEST);
    	lfincSM.setColumns(3);
    	lfincSM.setBounds(90,40,100,20,0.5);
    	lfincSM.setText(IJ.d2s(SmIncPxls,2));
    	lfincSM.addActionListener(this);

    	String[] butNames = {"N","E","S","W"};
    	int[] butX = {30,60,30,0};
    	for (int i=0; i<4; i++) {
    		butMove[i] = new Button(butNames[i]);
    		butMove[i].setBounds(butX[i],butX[3-i],30,30);
    		butMove[i].addActionListener(this);
    	}
        
    	Label labView = new Label("PROFILE");
    	labView.setBounds(0,0,300,20);
    	
    	bgcorrMd = new Choice();
    	bgcorrMd.setBounds(100,10,150,20);
    	for (int i=0; i<LSprocessor.BG_MODES.length; i++) bgcorrMd.addItem(LSprocessor.BG_MODES[i]);
    	bgcorrMd.select(ls.bgCorrIndex);
    	bgcorrMd.addItemListener(this);

    	bgcorrConst = new TextField();
    	bgcorrConst.setBounds(260,10,40,20);
    	boolean isActive = (ls.bgCorrIndex==LSprocessor.BG_CONST)?true:false;
//    	isActive = true;
    	bgcorrConst.setEnabled(isActive);
		bgcorrConst.setText(IJ.d2s(0,0));
		bgcorrConst.addActionListener(this);
    	if (isActive) {
//    		bgcorrConst.setText(IJ.d2s(0));
    		ls.regions[0].mode = 0;
    		ls.regions[2].mode = 0;
    	}
    	
    	String[] labMode = {"   Maximum","   Median","   Average","   Minimum"};
    	int mode;
    	for (int i=0; i<ls.regions.length; i++) {
    		ls.regions[i].makePanel((i==2)?140:40,100,4);
    		ls.regions[i].addItemListener(this);
    		ls.regions[i].p.setLocation(100+40*i,20);
    		mode = ls.regions[i].mode;
    		for (int j=0; j<LSRegion.MODES.length; j++) {
    			if (i==2) ls.regions[i].views[j].setLabel(labMode[j]);
    			if (mode<=LSRegion.MODES[j]) {
    				ls.regions[i].views[j].setState(true);
    				mode-=LSRegion.MODES[j];
    			}
    			else ls.regions[i].views[j].setState(false);
    		}
    	}
    	
    	Label labBgCorr = new Label("Corrected LS = ");
    	labBgCorr.setBounds(0,10,100,20);

        Label labInfo = new Label("LineScan Information:");
        labInfo.setBounds(bd,bd+470,w1,20);
        
        taInfo = new TextArea();
        taInfo.setBounds(bd,bd+490,w1,100);
        taInfo.setText(ls.lsInfo);
        
    	butLoad = new Button(" Load ");
    	butLoad.setBounds(10,10,80,30);
    	butLoad.addActionListener(this);
    	
    	butSave = new Button(" Save ");
    	butSave.setBounds(110,10,80,30);
    	butSave.addActionListener(this);
    	
    	butDDecon = new Button(" Do DDecon ");
    	butDDecon.setBounds(210,10,80,30);
    	butDDecon.addActionListener(this);
    	
    	// Add all the panels together
        this.add(masterPanel);
        
        masterPanel.add(panCont);
	        panCont.add(panSR);
		    	panSR.add(panSRw);
			    	panSRw.add(desc);
			    	for (int i=0; i<ls.regions.length; i++) panSRw.add(lfRegW[i].label); // Region Width Labels
			    	for (int i=0; i<ls.regions.length; i++) panSRw.add(lfRegW[i]); // Region Width Fields
			    	
		    	panSR.add(panSRi);
			    	panSRi.add(labSRLength); // Length of line
			    	panSRi.add(labSRWidth); // Width of line
			    	panSRi.add(labSRAngle); // Line Angle
	
	        panCont.add(panLineInfo);
		    	panLineInfo.add(labA);
		    	panLineInfo.add(labD);
		    	panLineInfo.add(labE);
				panLineInfo.add(labCP[0]);
				panLineInfo.add(labCP[1]);
		    	panLineInfo.add(panCP);
			        panCP.add(labCPx); // Control Point Label
					panCP.add(cbMoveLine[0]); // Control Point checkbox
					panCP.add(cbMoveLine[1]); // Control Point checkbox
	
		    	panLineInfo.add(panRF);
			    	panRF.add(labRF); // Reference frame label
			    	panRF.add(rbRefFrame0); // Reference frame Radio Button
			    	panRF.add(rbRefFrame1); // Reference frame Radio Button
			    	
	        panCont.add(panAdj);
	        	panAdj.add(panScr);
		        	panScr.add(lfSkewA); // SkewAngle Field
		        	panScr.add(lfSkewA.label); // SkewAngle label
		        	panScr.add(butSkewL); // SkewAngle Adj. button
		        	panScr.add(butSkewR); // SkewAngle Adj. button
		        	panScr.add(lfincLG); // large inc move Field
		        	panScr.add(lfincLG.label); // large inc move label
		        	panScr.add(lfincSM); // small inc move Field
		        	panScr.add(lfincSM.label); // small inc move label
		        	panScr.add(panScrBut);
			        	for (int i=0; i<4; i++) panScrBut.add(butMove[i]); // movement buttons
			        	
	    	panCont.add(panProfile);
		    	panProfile.add(panView);
			    	panView.add(labView); // profile label
			    	for (int i=0; i<3; i++) panView.add(ls.regions[i].p); // view panels
//			    	for (int i=0; i<3; i++) panView.add(region[i]); // view panels
	
		    	panProfile.add(panCalc);
			    	panCalc.add(labBgCorr); // bgCorr Label
			    	panCalc.add(bgcorrMd); // bgCorr Choice
			    	panCalc.add(bgcorrConst); // bgConst Field
//			    	if (isActive) panCalc.add(bgcorrConst); // bgConst Field
	
	        panCont.add(labInfo); // Information Label
	        panCont.add(taInfo); // Information Field
	        panCont.add(panActs);
		    	panActs.add(butLoad); // Load subImage button
		    	panActs.add(butSave); // Save subImage button
		    	panActs.add(butDDecon); // Export to DDecon button
	
        masterPanel.add(ImagePanel);
	        ImagePanel.add(panSubImage);
	        	panSubImage.add(cans[0]);
	        	
	        ImagePanel.add(panRawLS);
	        	panRawLS.add(cans[1]);
	        	
	        ImagePanel.add(panCorrLS);
	        	panCorrLS.add(cans[2]);
        
    }
    
/*    private Panel makeMasterPanel() {
        int bd = 5;
        int w1 = 300;
        int w2 = cans[1].getWidth();
		int subH = (int)(cans[0].getHeight()*subScale);
        int ic2h = cans[1].getHeight();
        int wt = w1+w2+6*bd;
        int ht = subH+2*ic2h+8*bd;
        Panel mp = new Panel();
        mp.setLayout(null);
        mp.setBackground(Color.darkGray);
        mp.setSize(wt,ht);
        panCont = makeControlPanel();
        panCont.setBounds(0,0,w1+2*bd,ht);
        mp.add(panCont);
        ImagePanel = makeImagePanel();
        ImagePanel.setBounds(w1+2*bd,0,w2+4*bd,ht);
        mp.add(ImagePanel);
        return mp;
    }
*/    
/*    private Panel makeControlPanel() {
        int bd = 5;
        int w1 = 300;
        int h1;
        int x1 = bd;
        int y1 = bd;
		int subH = (int)(cans[0].getHeight()*subScale);
        int ic2h = cans[1].getHeight();
        int wt = w1+2*bd;
        int ht = subH+2*ic2h+8*bd;

        Panel cp = new Panel();
        cp.setLayout(null);
        cp.setBackground(Color.gray);
        cp.setSize(wt,ht);

        Panel a = makeSubRegionInfoPanel();
        h1 = 100;
        a.setBounds(x1,y1,w1,h1);
        a.setBackground(Color.gray);
        cp.add(a);

        Panel b = makeLineInfoPanel();
        y1+=h1;
        h1 = 100;
        b.setBounds(x1,y1,w1,h1);
        b.setBackground(Color.lightGray);
        cp.add(b);

        Panel c = makeAdjustmentPanel();
        y1+=h1;
        h1 = 100;
        c.setBounds(x1,y1,w1,h1);
        c.setBackground(Color.gray);
        cp.add(c);

        Panel d = makeProfilePanel();
        y1+=h1;
        h1 = 170;
        d.setBounds(x1,y1,w1,h1);
        d.setBackground(Color.lightGray);
        cp.add(d);

        y1+=h1;
        h1 = 20;
        Label labInfo = new Label("LineScan Information:");
        labInfo.setBounds(x1,y1,w1,h1);
        cp.add(labInfo);
        
        y1+=h1;
        h1 = 100;
        taInfo = new TextArea();
        taInfo.setBounds(x1,y1,w1,h1);
        taInfo.setText("");
        cp.add(taInfo);
        
        Panel e = makeActionPanel();
        y1+=h1;
        h1 = 50;
        e.setBounds(x1,y1,w1,h1);
        e.setBackground(Color.gray);
        cp.add(e);
        return cp;
    }
*/    
/*    private Panel makeImagePanel() {
        int bd = 5;
        int w2 = cans[1].getWidth();
        int x2 = bd; //2*bd;//w1+2*bd;
        int y2 = bd;
		int subW = (int)(cans[0].getWidth()*subScale);
		int subH = (int)(cans[0].getHeight()*subScale);
        int ic2h = cans[1].getHeight();
        int ht = subH+2*ic2h+8*bd;//subH+2*ic2h+8*bd;
        ImagePanel = new Panel();
        ImagePanel.setLayout(null);
        ImagePanel.setBackground(Color.gray);
        ImagePanel.setSize(w2+4*bd,ht);
        panSubImage = new Panel(); // Subimage
        panRawLS = new Panel(); // Raw Linescans
        panCorrLS = new Panel(); // Corrected Linescan
        cans[0].setDrawingSize(subW,subH);
        cans[0].setMagnification(subScale);
        cans[0].setLocation(50+bd,bd);
        panSubImage.setBounds(x2,y2,w2+2*bd,subH+2*bd);
        panSubImage.setBackground(Color.darkGray);
        panSubImage.add(cans[0]);
        y2+=subH+2*bd;
        cans[1].setLocation(bd,bd);
        panRawLS.setBounds(x2,y2,w2+2*bd,ic2h+2*bd);
        panRawLS.setBackground(Color.darkGray);
        panRawLS.add(cans[1]);
        y2+=ic2h+2*bd;
        cans[2].setLocation(bd,bd);
        panCorrLS.setBounds(x2,y2,w2+2*bd,ic2h+2*bd);
        panCorrLS.setBackground(Color.darkGray);
        panCorrLS.add(cans[2]);
        ImagePanel.add(panSubImage);
        ImagePanel.add(panRawLS);
        ImagePanel.add(panCorrLS);
        return ImagePanel;
    }
*/    
/*    private Panel makeSubRegionInfoPanel() {
    	Panel panSR = new Panel();
    	panSR.setSize(300,100);
    	panSR.setLayout(null);
    	
    	Panel a = makeSRwidthsPanel();
    	a.setLocation(0,0);
    	panSR.add(a);

    	Panel b = makeSRinfoPanel();
    	b.setLocation(120,0);
    	panSR.add(b);
    	return panSR;
    }
*/    
/*    private Panel makeSRwidthsPanel() {
    	Panel panSRw = new Panel();
    	panSRw.setSize(100,100);
    	panSRw.setLayout(null);
    	Label desc = new Label("Width");
    	desc.setBounds(50,0,50,20);
    	panSRw.add(desc);

    	Label labBG1 = new Label("BG1");
    	labBG1.setBounds(0,20,50,20);
    	panSRw.add(labBG1);
    	tfBG1w = new TextField(2);
    	tfBG1w.setBounds(50,20,50,20);
//    	tfBG1w.setText(IJ.d2s(ls.regions[0].rect.height,0));
    	tfBG1w.setText(IJ.d2s(lsi.bg1w,0));
    	tfBG1w.addActionListener(this);
    	panSRw.add(tfBG1w);
    	
    	Label labLSr = new Label("LSraw");
    	labLSr.setBounds(0,40,50,20);
    	panSRw.add(labLSr);
    	tfLSw = new TextField(2);
    	tfLSw.setBounds(50,40,50,20);
//    	tfLSw.setText(IJ.d2s(ls.regions[1].rect.height,0));
    	tfLSw.setText(IJ.d2s(lsi.rawLSw,0));
    	tfLSw.addActionListener(this);
    	panSRw.add(tfLSw);
    	
    	Label labBG2 = new Label("BG2");
    	labBG2.setBounds(0,60,50,20);
    	panSRw.add(labBG2);
    	tfBG2w = new TextField(2);
    	tfBG2w.setBounds(50,60,50,20);
//    	tfBG2w.setText(IJ.d2s(ls.regions[2].rect.height,0));
    	tfBG2w.setText(IJ.d2s(lsi.bg2w,0));
    	tfBG2w.addActionListener(this);
    	panSRw.add(tfBG2w);
    	return panSRw;
    }
*/    
/*    private Panel makeSRinfoPanel() {
    	Panel panSRi = new Panel();
    	panSRi.setSize(180,100);
    	panSRi.setLayout(null);
//    	labSRLength = new Label(" Total Length = "+ls.lsLength+" pixels");
    	labSRLength = new Label(" Total Length = "+lsi.lsLength+" pixels");
    	labSRLength.setBounds(0,20,180,20);
    	panSRi.add(labSRLength);
//    	labSRWidth = new Label(" Total Width = "+ls.lsWidth+" pixels");
    	labSRWidth = new Label(" Total Width = "+lsi.lsWidth+" pixels");
    	labSRWidth.setBounds(0,40,180,20);
    	panSRi.add(labSRWidth);
//    	labSRAngle = new Label(" Line Angle = "+IJ.d2s((180*ls.lsAngle/Math.PI),2)+String.valueOf(IJ.degreeSymbol));
    	labSRAngle = new Label(" Line Angle = "+IJ.d2s((180*lsi.lsAngle/Math.PI),2)+String.valueOf(IJ.degreeSymbol));
    	labSRAngle.setBounds(0,60,180,20);
    	panSRi.add(labSRAngle);
    	return panSRi;
    }
*/  
/*    private Panel makeLineInfoPanel() {
    	panLineInfo = new Panel();
    	panLineInfo.setSize(300,100);
    	panLineInfo.setLayout(null);
    	Label labA = new Label("ROI");
    	labA.setBounds(0,20,40,20);
    	panLineInfo.add(labA);
   	
    	Label labD = new Label("Start");
    	labD.setBounds(0,40,40,20);
    	panLineInfo.add(labD);
    	
    	Label labE = new Label("End");
    	labE.setBounds(0,60,40,20);
    	panLineInfo.add(labE);
    	
    	double[] x = new double[2];
//    	x[0] = ls.line.x1d;
//    	x[1] = ls.line.x2d;
    	x[0] = lsi.line.x1d;
    	x[1] = lsi.line.x2d;
    	double[] y = new double[2];
//    	y[0] = ls.line.y1d;
//    	y[1] = ls.line.y2d;
    	y[0] = lsi.line.y1d;
    	y[1] = lsi.line.y2d;
    	
    	for (int i=0; i<labCP.length; i++) {
    		labCP[i] = new Label("("+IJ.d2s(x[i],2)+","+IJ.d2s(y[i],2)+")");
    		labCP[i].setBounds(40,40+20*i,100,20);
    		panLineInfo.add(labCP[i]);
    	}
    	
    	Panel a = makeMovePointsPanel();
    	a.setLocation(140,0);
    	panLineInfo.add(a);

    	Panel b = makeRefFramePanel();
    	b.setLocation(200,0);
    	panLineInfo.add(b);

    	return panLineInfo;
    }
*/    
/*    private Panel makeAdjustmentPanel() {
    	panAdj = new Panel();
    	panAdj.setLayout(null);
    	panAdj.setSize(300,100);
    	Panel c = makeScrollPanel();
    	c.setLocation(0,0);
    	panAdj.add(c);
    	return panAdj;
    }
*/    
/*    private Panel makeMovePointsPanel() {
    	Panel panCP = new Panel();
    	panCP.setLayout(null);
    	panCP.setSize(50,100);
    	
    	Label labCP = new Label("-- Pts --");
    	labCP.setBounds(0,20,50,20);
    	panCP.add(labCP);
    	
    	for (int i=0; i<cbMoveLine.length; i++) {
    		cbMoveLine[i] = new Checkbox(" "+(i+1)+" ",null,true);
    		cbMoveLine[i].setBounds(0,40+20*i,50,20);
    		panCP.add(cbMoveLine[i]);
    	}
    	
    	return panCP;
    }
*/    
/*    private Panel makeRefFramePanel() {
    	Panel panRF = new Panel();
    	panRF.setLayout(null);
    	panRF.setSize(100,100);
    	
    	Label labRF = new Label("-- Ref Frame --");
    	labRF.setBounds(0,20,100,20);
    	panRF.add(labRF);
    	
    	mvRefFrame = new CheckboxGroup();
    	rbRefFrame0 = new Checkbox(" Original ",mvRefFrame,isOrigRefFrame);
    	rbRefFrame0.setBounds(0,40,100,20);
    	rbRefFrame0.addItemListener(this);
    	panRF.add(rbRefFrame0);
    	rbRefFrame1 = new Checkbox(" Subregion ",mvRefFrame,!isOrigRefFrame);
    	rbRefFrame1.setBounds(0,60,100,20);
    	rbRefFrame1.addItemListener(this);
    	panRF.add(rbRefFrame1);
    	return panRF;
    }
*/    
/*    private Panel makeScrollPanel() {
    	Panel panScr = new Panel();
    	panScr.setLayout(null);
    	panScr.setSize(300,100);
    	
    	lfSkewA = new LField("Skew",LField.NORTH);
    	lfSkewA.setColumns(2);
    	lfSkewA.setBounds(5,5,70,40,0.6);
//    	lfSkewA.setText(IJ.d2s(ls.skewAngle));
    	lfSkewA.setText(IJ.d2s(lsi.skewAngle));
    	lfSkewA.addActionListener(this);
    	panScr.add(lfSkewA);
    	panScr.add(lfSkewA.label);
    	
    	butSkewL = new Button("L");
    	butSkewL.setBounds(5,50,30,30);
    	butSkewL.addActionListener(this);
    	panScr.add(butSkewL);
    	
    	butSkewR = new Button("R");
    	butSkewR.setBounds(45,50,30,30);
    	butSkewR.addActionListener(this);
    	panScr.add(butSkewR);
    	
    	lfincLG = new LField("LG inc",LField.WEST);
    	lfincLG.setColumns(2);
    	lfincLG.setBounds(90,20,100,20,0.5);
    	lfincLG.setText(IJ.d2s(LgIncPxls,0));
    	lfincLG.addActionListener(this);
    	panScr.add(lfincLG);
    	panScr.add(lfincLG.label);

    	lfincSM = new LField("SM inc",LField.WEST);
    	lfincSM.setColumns(3);
    	lfincSM.setBounds(90,40,100,20,0.5);
    	lfincSM.setText(IJ.d2s(SmIncPxls,2));
    	lfincSM.addActionListener(this);
    	panScr.add(lfincSM);
    	panScr.add(lfincSM.label);

    	Panel panScrBut = new Panel();
    	panScrBut.setLayout(null);
    	panScrBut.setBounds(200,5,90,90);
    	
    	butMoveN = new Button("N");
    	butMoveN.setBounds(30,0,30,30);
    	butMoveN.addActionListener(this);
    	panScrBut.add(butMoveN);
    	
    	butMoveW = new Button("W");
    	butMoveW.setBounds(0,30,30,30);
    	butMoveW.addActionListener(this);
    	panScrBut.add(butMoveW);
    	
    	butMoveE = new Button("E");
    	butMoveE.setBounds(60,30,30,30);
    	butMoveE.addActionListener(this);
    	panScrBut.add(butMoveE);
    	
     	butMoveS = new Button("S");
    	butMoveS.setBounds(30,60,30,30);
    	butMoveS.addActionListener(this);
    	panScrBut.add(butMoveS);
    	
    	panScr.add(panScrBut);
    	return panScr;
    }
*/    
/*    private Panel makeProfilePanel() {
    	panProfile = new Panel();
    	panProfile.setSize(300,170);
    	panProfile.setLayout(null);
    	Panel a = makeProfileViewPanel();
    	a.setLocation(0,0);
    	panProfile.add(a);
    	Panel b = makeProfileCalcPanel();
    	b.setLocation(0,120);
    	panProfile.add(b);
    	return panProfile;
    }
*/    
/*    private Panel makeProfileViewPanel() {
    	Panel panView = new Panel();
    	panView.setLayout(null);
    	panView.setBounds(0,0,300,120);
    	Label label = new Label("PROFILE");
    	label.setBounds(0,0,300,20);
    	panView.add(label);
    	
    	region = new ViewPanel [3];
    	String[] text = {"BG1","LSr","BG2"};
    	int[] loc = {100,140,180};
    	for (int i=0; i<3; i++) {
    		region[i] = new ViewPanel(((i==2)?140:40),100,4,this);
    		region[i].label.setText(text[i]);
    		region[i].setLocation(loc[i],20);
    		panView.add(region[i]);
    	}
//    	boolean[][] rvals = new boolean[3][4];
//    	int[] type = {LSRegion.MAX,LSRegion.MED,LSRegion.AVE,LSRegion.MIN};
    	for (int i=0; i<ls.regions.length; i++) {
    		for (int j=0; j<4; j++) {
//    			rvals[i][j] = (ls.regions[i].mode&type[j])!=0;
    		}
    	}
    	boolean[] r0vals = ls.decodeMode(0); // default view BG1
    	boolean[] r1vals = ls.decodeMode(1); // default view LSr
    	boolean[] r2vals = ls.decodeMode(2); // default view BG2
    	String[] labels = {"   Maximum","   Median","   Average","   Minimum"};
    	for (int i=0; i<4; i++) {
//        	region[0].views[i].setState(rvals[0][i]);
//        	region[1].views[i].setState(rvals[1][i]);
//        	region[2].views[i].setState(rvals[2][i]);
        	region[0].views[i].setState(r0vals[i]);
        	region[1].views[i].setState(r1vals[i]);
        	region[2].views[i].setState(r2vals[i]);
    		region[2].views[i].setLabel(labels[i]);
    	}
    	return panView;
    }
*/    
/*    private Panel makeProfileCalcPanel() {
    	Panel panCalc = new Panel();
    	panCalc.setLayout(null);
    	panCalc.setSize(300,50);
    	Label labA = new Label("Corrected LS = ");
    	labA.setBounds(0,10,100,20);
    	panCalc.add(labA);
    	int index = lsi.corrcalc;
    	bgcorrMd = new Choice();
    	bgcorrMd.setBounds(100,10,150,20);
    	for (int i=0; i<LSprocessor.BG_MODES.length; i++) {
    		bgcorrMd.addItem(LSprocessor.BG_MODES[i]);
    	}
//    	bgcorrMd.select(ls.bgCorrIndex);
    	bgcorrMd.select(index);
    	bgcorrMd.addItemListener(this);
    	panCalc.add(bgcorrMd);
    	bgcorrConst = new TextField();
    	bgcorrConst.setBounds(260,10,40,20);
    	bgcorrConst.setEnabled(false);
    	panCalc.add(bgcorrConst);
    	return panCalc;
    }
*/    
/*    private Panel makeActionPanel() {
    	panActs = new Panel();
    	panActs.setLayout(null);
    	panActs.setSize(300,50);
    	
    	butLoad = new Button(" Load ");
    	butLoad.setBounds(10,10,80,30);
    	butLoad.addActionListener(this);
    	panActs.add(butLoad);
    	
    	butSave = new Button(" Save ");
    	butSave.setBounds(110,10,80,30);
    	butSave.addActionListener(this);
    	panActs.add(butSave);
    	
    	butDDecon = new Button(" Export ");
    	butDDecon.setBounds(210,10,80,30);
    	butDDecon.addActionListener(this);
    	panActs.add(butDDecon);
        return panActs;
    }
*/
}
