/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.geom.Point2D;

public class PSFModel {

	static final int MIN_LENGTH = 31;
	public String title;
	public String desc;
	public double focusParam;
	public boolean pState;
	public Point2D.Double[] pixels;
	
	public PSFModel(String title, String desc, double[] params, boolean[] pStates) {
		this.title = title;
		this.desc = desc;
		this.focusParam = params[0];
		this.pState = pStates[0];
		this.pixels = makePixels(focusParam);
	}
	
	/** Uses a normal distribution to make a centered 1-D PSF.
	 * @param n: number of data points to return.  If n is even, n-1 points are returned. */
	public Point2D.Double[] makePixels(double focusParam) {
		int n = Math.max((int)(focusParam*8),PSFModel.MIN_LENGTH);
		if (n % 2 == 0) n--;
		Point2D.Double[] pixels = new Point2D.Double [n];
		int mu = (int)((n-1)/2);
		double x,y;
		for (int i=0; i<n;i++) {
			x = i-mu;
			y = Math.exp(-(i-mu)*(i-mu)/(2*focusParam*focusParam))/(focusParam*Math.sqrt(2*Math.PI));
			pixels[i] = new Point2D.Double(x,y);
		}
		return pixels;
	}
}
